@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkCellDataToPointData.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-  This software is distributed WITHOUT ANY WARRANTY; without even
-  the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-  PURPOSE.  See the above copyright notice for more information.
-
-  =========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkCellDataToPointData.h"
 
 #include "vtkAbstractCellLinks.h"
@@ -30,6 +18,7 @@
 #include "vtkSMPTools.h"
 #include "vtkSmartPointer.h"
 #include "vtkStaticCellLinks.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkStructuredGrid.h"
 #include "vtkUniformGrid.h"
 #include "vtkUnsignedIntArray.h"
@@ -41,7 +30,8 @@
 
 #define VTK_MAX_CELLS_PER_POINT 4096
 
-vtkStandardNewMacro(vtkCellDataToPointData);
+VTK_ABI_NAMESPACE_BEGIN
+vtkObjectFactoryNewMacro(vtkCellDataToPointData);
 
 namespace
 {
@@ -52,67 +42,93 @@ namespace
 template <typename TCellLinks>
 struct UnstructuredDataCD2PD
 {
-  vtkIdType NumPts;
-  vtkDataSetAttributes* InDA;
-  vtkPointData* OutDA;
   TCellLinks* Links;
-  ArrayList* Arrays;
+  ArrayList Arrays;
 
-  UnstructuredDataCD2PD(
-    vtkIdType numPts, vtkDataSetAttributes* inDA, vtkPointData* outDA, TCellLinks* links)
-    : NumPts(numPts)
-    , InDA(inDA)
-    , OutDA(outDA)
-    , Links(links)
+  UnstructuredDataCD2PD(vtkIdType numPts, vtkCellData* inDA, vtkPointData* outDA, TCellLinks* links)
+    : Links(links)
   {
-    this->Arrays = new ArrayList;
-    this->Arrays->AddArrays(numPts, inDA, outDA);
+    this->Arrays.AddArrays(numPts, inDA, outDA);
   }
-  ~UnstructuredDataCD2PD() { delete this->Arrays; }
 
-  void operator()(vtkIdType ptId, vtkIdType endPtId)
+  void operator()(vtkIdType beginPointId, vtkIdType endPointId)
   {
     vtkIdType ncells;
-
-    for (; ptId < endPtId; ++ptId)
+    for (vtkIdType pointId = beginPointId; pointId < endPointId; ++pointId)
     {
-      if ((ncells = this->Links->GetNcells(ptId)) > 0)
+      if ((ncells = this->Links->GetNcells(pointId)) > 0)
       {
-        auto cells = this->Links->GetCells(ptId);
-        this->Arrays->Average(ncells, cells, ptId);
+        auto cells = this->Links->GetCells(pointId);
+        this->Arrays.Average(ncells, cells, pointId);
+      }
+      else
+      {
+        this->Arrays.AssignNullValue(pointId);
       }
-    }
-  }
-
-  void Execute()
-  {
-    if (this->NumPts > 0)
-    {
-      vtkSMPTools::For(0, this->NumPts, *this);
     }
   }
 };
 
-// Take care of dispatching to the functor.
-void FastUnstructuredData(
-  vtkIdType numPts, vtkAbstractCellLinks* links, vtkDataSetAttributes* cfl, vtkPointData* pd)
+//------------------------------------------------------------------------------
+// Take care of dispatching to the functor using an abstract cell links.
+void FastUnstructuredDataACL(
+  vtkIdType numPts, vtkAbstractCellLinks* links, vtkCellData* cfl, vtkPointData* pd)
 {
   assert(links != nullptr);
   if (auto staticCellLinks = vtkStaticCellLinks::SafeDownCast(links))
   {
     UnstructuredDataCD2PD<vtkStaticCellLinks> cd2pd(numPts, cfl, pd, staticCellLinks);
-    cd2pd.Execute();
+    vtkSMPTools::For(0, numPts, cd2pd);
   }
   else // vtkCellLinks
   {
     auto cellLinks = vtkCellLinks::SafeDownCast(links);
     UnstructuredDataCD2PD<vtkCellLinks> cd2pd(numPts, cfl, pd, cellLinks);
-    cd2pd.Execute();
+    vtkSMPTools::For(0, numPts, cd2pd);
   }
 }
 
 //------------------------------------------------------------------------------
-// Helper template function that implement the major part of the algorithm
+// Take care of dispatching to the functor using a static cell links template instance.
+template <typename TInput>
+void FastUnstructuredDataSCLT(
+  vtkIdType connectivitySize, TInput* input, vtkCellData* cfl, vtkPointData* pd)
+{
+  const auto numberOfPoints = input->GetNumberOfPoints();
+  const auto numberOfCells = input->GetNumberOfCells();
+  auto linksType =
+    vtkAbstractCellLinks::ComputeType(numberOfPoints - 1, numberOfCells - 1, connectivitySize);
+  // build the appropriate static cell links template instance
+  if (linksType == vtkAbstractCellLinks::STATIC_CELL_LINKS_USHORT)
+  {
+    using TCellLinks = vtkStaticCellLinksTemplate<unsigned short>;
+    TCellLinks cellLinks;
+    cellLinks.BuildLinks(input);
+    UnstructuredDataCD2PD<TCellLinks> cd2pd(numberOfPoints, cfl, pd, &cellLinks);
+    vtkSMPTools::For(0, numberOfPoints, cd2pd);
+  }
+#ifdef VTK_USE_64BIT_IDS
+  else if (linksType == vtkAbstractCellLinks::STATIC_CELL_LINKS_UINT)
+  {
+    using TCellLinks = vtkStaticCellLinksTemplate<unsigned int>;
+    TCellLinks cellLinks;
+    cellLinks.BuildLinks(input);
+    UnstructuredDataCD2PD<TCellLinks> cd2pd(numberOfPoints, cfl, pd, &cellLinks);
+    vtkSMPTools::For(0, numberOfPoints, cd2pd);
+  }
+#endif
+  else
+  {
+    using TCellLinks = vtkStaticCellLinksTemplate<vtkIdType>;
+    TCellLinks cellLinks;
+    cellLinks.BuildLinks(input);
+    UnstructuredDataCD2PD<TCellLinks> cd2pd(numberOfPoints, cfl, pd, &cellLinks);
+    vtkSMPTools::For(0, numberOfPoints, cd2pd);
+  }
+}
+
+//------------------------------------------------------------------------------
+// Helper template function that implements the major part of the algorithm
 // which will be expanded by the vtkTemplateMacro. The template function is
 // provided so that coverage test can cover this function. This approach is
 // slow: it's non-threaded; uses a slower vtkDataSet API; and most
@@ -123,7 +139,7 @@ struct Spread
   template <typename SrcArrayT, typename DstArrayT>
   void operator()(SrcArrayT* const srcarray, DstArrayT* const dstarray, vtkDataSet* const src,
     vtkUnsignedIntArray* const num, vtkIdType ncells, vtkIdType npoints, vtkIdType ncomps,
-    int highestCellDimension, int contributingCellOption) const
+    int highestCellDimension, int contributingCellOption, vtkCellDataToPointData* filter) const
   {
     // Both arrays will have the same value type:
     using T = vtk::GetAPIType<SrcArrayT>;
@@ -133,13 +149,19 @@ struct Spread
 
     const auto srcTuples = vtk::DataArrayTupleRange(srcarray);
     auto dstTuples = vtk::DataArrayTupleRange(dstarray);
+    vtkIdType checkAbortInterval;
 
     // accumulate
     if (contributingCellOption != vtkCellDataToPointData::Patch)
     {
       vtkNew<vtkIdList> pointIds;
+      checkAbortInterval = std::min(ncells / 10 + 1, (vtkIdType)1000);
       for (vtkIdType cid = 0; cid < ncells; ++cid)
       {
+        if (cid % checkAbortInterval == 0 && filter->CheckAbort())
+        {
+          break;
+        }
         int dimension = vtkCellTypes::GetDimension(src->GetCellType(cid));
         if (dimension >= highestCellDimension)
         {
@@ -156,8 +178,14 @@ struct Spread
         }
       }
       // average
+
+      checkAbortInterval = std::min(npoints / 10 + 1, (vtkIdType)1000);
       for (vtkIdType pid = 0; pid < npoints; ++pid)
       {
+        if (pid % checkAbortInterval == 0 && filter->CheckAbort())
+        {
+          break;
+        }
         // guard against divide by zero
         if (unsigned int const denom = num->GetValue(pid))
         {
@@ -173,8 +201,13 @@ struct Spread
     { // compute over cell patches
       vtkNew<vtkIdList> cellsOnPoint;
       std::vector<T> data(4 * ncomps);
+      checkAbortInterval = std::min(npoints / 10 + 1, (vtkIdType)1000);
       for (vtkIdType pid = 0; pid < npoints; ++pid)
       {
+        if (pid % checkAbortInterval == 0 && filter->CheckAbort())
+        {
+          break;
+        }
         std::fill(data.begin(), data.end(), 0);
         T numPointCells[4] = { 0, 0, 0, 0 };
         // Get all cells touching this point.
@@ -227,44 +260,44 @@ public:
     vtkNew<vtkIdList> cellIds;
     cellIds->Allocate(8);
 
-    vtkIdType numPts = input->GetNumberOfPoints();
+    const vtkIdType numberOfPoints = input->GetNumberOfPoints();
 
-    vtkCellData* inputInCD = input->GetCellData();
-    vtkCellData* inCD;
+    vtkCellData* inCD = input->GetCellData();
     vtkPointData* outPD = output->GetPointData();
 
+    // Copy all existing cell fields into a temporary cell data array,
+    // unless the SelectCellDataArrays option is active.
+    vtkNew<vtkCellData> processedCellData;
     if (!filter->GetProcessAllArrays())
     {
-      inCD = vtkCellData::New();
-
       for (const auto& name : this->CellDataArrays)
       {
-        vtkAbstractArray* arr = inputInCD->GetAbstractArray(name.c_str());
-        if (arr == nullptr)
+        vtkAbstractArray* array = inCD->GetAbstractArray(name.c_str());
+        if (!array)
         {
           vtkWarningWithObjectMacro(filter, "cell data array name not found.");
           continue;
         }
-        inCD->AddArray(arr);
+        processedCellData->AddArray(array);
       }
     }
     else
     {
-      inCD = inputInCD;
+      processedCellData->ShallowCopy(inCD);
     }
 
-    outPD->InterpolateAllocate(inCD, numPts);
+    outPD->InterpolateAllocate(processedCellData, numberOfPoints);
 
     double weights[8];
 
-    int abort = 0;
-    vtkIdType progressInterval = numPts / 20 + 1;
-    for (vtkIdType ptId = 0; ptId < numPts && !abort; ptId++)
+    bool abort = false;
+    vtkIdType progressInterval = numberOfPoints / 20 + 1;
+    for (vtkIdType ptId = 0; ptId < numberOfPoints && !abort; ptId++)
     {
       if (!(ptId % progressInterval))
       {
-        filter->UpdateProgress(static_cast<double>(ptId) / numPts);
-        abort = filter->GetAbortExecute();
+        filter->UpdateProgress(static_cast<double>(ptId) / numberOfPoints);
+        abort = filter->CheckAbort();
       }
       input->GetPointCells(ptId, allCellIds);
       cellIds->Reset();
@@ -287,7 +320,7 @@ public:
         {
           weights[cellId] = weight;
         }
-        outPD->InterpolatePoint(inCD, ptId, cellIds, weights);
+        outPD->InterpolatePoint(processedCellData, ptId, cellIds, weights);
       }
       else
       {
@@ -295,11 +328,6 @@ public:
       }
     }
 
-    if (!filter->GetProcessAllArrays())
-    {
-      inCD->Delete();
-    }
-
     return 1;
   }
 };
@@ -311,6 +339,7 @@ vtkCellDataToPointData::vtkCellDataToPointData()
   this->PassCellData = false;
   this->ContributingCellOption = vtkCellDataToPointData::All;
   this->ProcessAllArrays = true;
+  this->PieceInvariant = true;
   this->Implementation = new Internals();
 }
 
@@ -356,15 +385,28 @@ void vtkCellDataToPointData::ClearCellDataArrays()
   this->Implementation->CellDataArrays.clear();
 }
 
+//------------------------------------------------------------------------------
+vtkIdType vtkCellDataToPointData::GetNumberOfCellArraysToProcess()
+{
+  return static_cast<vtkIdType>(this->Implementation->CellDataArrays.size());
+}
+
+//------------------------------------------------------------------------------
+void vtkCellDataToPointData::GetCellArraysToProcess(const char* names[])
+{
+  for (const auto& n : this->Implementation->CellDataArrays)
+  {
+    *names = n.c_str();
+    ++names;
+  }
+}
+
 //------------------------------------------------------------------------------
 int vtkCellDataToPointData::RequestData(
   vtkInformation*, vtkInformationVector** inputVector, vtkInformationVector* outputVector)
 {
-  vtkInformation* info = outputVector->GetInformationObject(0);
-  vtkDataSet* output = vtkDataSet::SafeDownCast(info->Get(vtkDataObject::DATA_OBJECT()));
-
-  vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
-  vtkDataSet* input = vtkDataSet::SafeDownCast(inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkDataSet* input = vtkDataSet::GetData(inputVector[0]);
+  vtkDataSet* output = vtkDataSet::GetData(outputVector);
 
   vtkDebugMacro(<< "Mapping cell data to point data");
 
@@ -375,17 +417,19 @@ int vtkCellDataToPointData::RequestData(
     return this->RequestDataForUnstructuredData(nullptr, inputVector, outputVector);
   }
 
-  vtkDebugMacro(<< "Mapping cell data to point data");
-
   // First, copy the input to the output as a starting point
   output->CopyStructure(input);
 
+  vtkPointData* inPD = input->GetPointData();
+  vtkCellData* inCD = input->GetCellData();
+  vtkPointData* outPD = output->GetPointData();
+  vtkCellData* outCD = output->GetCellData();
+
   // Pass the point data first. The fields and attributes
   // which also exist in the cell data of the input will
   // be over-written during CopyAllocate
-  output->GetPointData()->CopyGlobalIdsOff();
-  output->GetPointData()->PassData(input->GetPointData());
-  output->GetPointData()->CopyFieldOff(vtkDataSetAttributes::GhostArrayName());
+  outPD->PassData(inPD);
+  outPD->CopyFieldOff(vtkDataSetAttributes::GhostArrayName());
 
   if (input->GetNumberOfPoints() < 1)
   {
@@ -417,11 +461,46 @@ int vtkCellDataToPointData::RequestData(
 
   if (!this->PassCellData)
   {
-    output->GetCellData()->CopyAllOff();
-    output->GetCellData()->CopyFieldOn(vtkDataSetAttributes::GhostArrayName());
+    outCD->CopyAllOff();
+    outCD->CopyFieldOn(vtkDataSetAttributes::GhostArrayName());
   }
-  output->GetCellData()->PassData(input->GetCellData());
-  output->GetFieldData()->PassData(input->GetFieldData());
+  outCD->PassData(inCD);
+
+  return 1;
+}
+
+//------------------------------------------------------------------------------
+int vtkCellDataToPointData::RequestUpdateExtent(
+  vtkInformation*, vtkInformationVector** inputVector, vtkInformationVector* outputVector)
+{
+  if (!this->PieceInvariant)
+  {
+    // I believe the default input update extent
+    // is set to the input update extent.
+    return 1;
+  }
+
+  // Technically, this code is only correct for pieces extent types.  However,
+  // since this class is pretty inefficient for data types that use 3D extents,
+  // we'll punt on the ghost levels for them, too.
+
+  // get the info objects
+  vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
+
+  int piece = outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER());
+  int numPieces = outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES());
+  int ghostLevels = outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS());
+
+  if (numPieces > 1)
+  {
+    ++ghostLevels;
+  }
+
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER(), piece);
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES(), numPieces);
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS(), ghostLevels);
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::EXACT_EXTENT(), 1);
 
   return 1;
 }
@@ -433,6 +512,7 @@ void vtkCellDataToPointData::PrintSelf(ostream& os, vtkIndent indent)
 
   os << indent << "PassCellData: " << (this->PassCellData ? "On\n" : "Off\n");
   os << indent << "ContributingCellOption: " << this->ContributingCellOption << endl;
+  os << indent << "PieceInvariant: " << (this->PieceInvariant ? "On\n" : "Off\n");
 }
 
 //----------------------------------------------------------------------------
@@ -442,14 +522,12 @@ void vtkCellDataToPointData::PrintSelf(ostream& os, vtkIndent indent)
 int vtkCellDataToPointData::RequestDataForUnstructuredData(
   vtkInformation*, vtkInformationVector** inputVector, vtkInformationVector* outputVector)
 {
-  vtkDataSet* const src = vtkDataSet::SafeDownCast(
-    inputVector[0]->GetInformationObject(0)->Get(vtkDataObject::DATA_OBJECT()));
-  vtkDataSet* const dst = vtkDataSet::SafeDownCast(
-    outputVector->GetInformationObject(0)->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPointSet* input = vtkPointSet::GetData(inputVector[0]);
+  vtkPointSet* output = vtkPointSet::GetData(outputVector);
 
-  vtkIdType const ncells = src->GetNumberOfCells();
-  vtkIdType const npoints = src->GetNumberOfPoints();
-  if (ncells < 1 || npoints < 1)
+  const vtkIdType numberOfCells = input->GetNumberOfCells();
+  const vtkIdType numberOfPoints = input->GetNumberOfPoints();
+  if (numberOfCells < 1 || numberOfPoints < 1)
   {
     vtkDebugMacro(<< "No input data!");
     return 1;
@@ -459,35 +537,37 @@ int vtkCellDataToPointData::RequestDataForUnstructuredData(
 
   // First, copy the input structure (geometry and topology) to the output as
   // a starting point.
-  dst->CopyStructure(src);
-  vtkPointData* const opd = dst->GetPointData();
+  output->CopyStructure(input);
+
+  vtkCellData* inCD = input->GetCellData();
+  vtkPointData* outPD = output->GetPointData();
+  vtkCellData* outCD = output->GetCellData();
 
   // Pass the point data first. The fields and attributes which also exist in
   // the cell data of the input will be over-written during CopyAllocate
-  opd->CopyGlobalIdsOff();
-  opd->PassData(src->GetPointData());
-  opd->CopyFieldOff(vtkDataSetAttributes::GhostArrayName());
+  outPD->CopyGlobalIdsOff();
+  outPD->PassData(input->GetPointData());
+  outPD->CopyFieldOff(vtkDataSetAttributes::GhostArrayName());
 
   // Copy all existing cell fields into a temporary cell data array,
   // unless the SelectCellDataArrays option is active.
-  vtkSmartPointer<vtkCellData> processedCellData = vtkSmartPointer<vtkCellData>::New();
+  vtkNew<vtkCellData> processedCellData;
   if (!this->ProcessAllArrays)
   {
-    vtkCellData* processedCellDataTemp = src->GetCellData();
     for (const auto& name : this->Implementation->CellDataArrays)
     {
-      vtkAbstractArray* arr = processedCellDataTemp->GetAbstractArray(name.c_str());
-      if (arr == nullptr)
+      vtkAbstractArray* array = inCD->GetAbstractArray(name.c_str());
+      if (!array)
       {
         vtkWarningMacro("cell data array name not found.");
         continue;
       }
-      processedCellData->AddArray(arr);
+      processedCellData->AddArray(array);
     }
   }
   else
   {
-    processedCellData->ShallowCopy(src->GetCellData());
+    processedCellData->ShallowCopy(inCD);
   }
 
   // Remove all fields that are not a data array.
@@ -499,18 +579,15 @@ int vtkCellDataToPointData::RequestDataForUnstructuredData(
     }
   }
 
-  // Cell field list constructed from the filtered cell data array
-  vtkDataSetAttributes::FieldList cfl(1);
-  cfl.InitializeFieldList(processedCellData);
-  opd->InterpolateAllocate(processedCellData, npoints);
+  outPD->InterpolateAllocate(processedCellData, numberOfPoints);
 
   // Pass the input cell data to the output as appropriate.
   if (!this->PassCellData)
   {
-    dst->GetCellData()->CopyAllOff();
-    dst->GetCellData()->CopyFieldOn(vtkDataSetAttributes::GhostArrayName());
+    outCD->CopyAllOff();
+    outCD->CopyFieldOn(vtkDataSetAttributes::GhostArrayName());
   }
-  dst->GetCellData()->PassData(src->GetCellData());
+  outCD->PassData(inCD);
 
   // Now perform the averaging operation.
 
@@ -518,17 +595,41 @@ int vtkCellDataToPointData::RequestDataForUnstructuredData(
   // unstructured datasets. A common workflow requiring maximum performance.
   if (this->ContributingCellOption == vtkCellDataToPointData::All)
   {
-    if (auto uGrid = vtkUnstructuredGrid::SafeDownCast(src))
+    if (auto uGrid = vtkUnstructuredGrid::SafeDownCast(input))
     {
-      uGrid->BuildLinks();
-      FastUnstructuredData(npoints, uGrid->GetLinks(), processedCellData, opd);
+      if (uGrid->GetLinks()) // if links are present use them
+      {
+        uGrid->BuildLinks(); // ensure links are up to date
+        FastUnstructuredDataACL(numberOfPoints, uGrid->GetLinks(), processedCellData, outPD);
+      }
+      else // otherwise create links with the minimum size
+      {
+        vtkIdType connectivitySize = uGrid->GetCells()->GetNumberOfConnectivityIds();
+        FastUnstructuredDataSCLT(connectivitySize, uGrid, processedCellData, outPD);
+      }
       return 1;
     }
     else // polydata
     {
-      auto polyData = vtkPolyData::SafeDownCast(src);
-      polyData->BuildLinks();
-      FastUnstructuredData(npoints, polyData->GetLinks(), processedCellData, opd);
+      auto polyData = vtkPolyData::SafeDownCast(input);
+      if (polyData->GetLinks()) // if links are present use them
+      {
+        polyData->BuildLinks(); // ensure links are up to date
+        FastUnstructuredDataACL(numberOfPoints, polyData->GetLinks(), processedCellData, outPD);
+      }
+      else // otherwise create links with the minimum size
+      {
+        auto verts = polyData->GetVerts();
+        auto lines = polyData->GetLines();
+        auto polys = polyData->GetPolys();
+        auto strips = polyData->GetStrips();
+        vtkIdType connectivitySize = 0;
+        connectivitySize += verts ? verts->GetNumberOfConnectivityIds() : 0;
+        connectivitySize += lines ? lines->GetNumberOfConnectivityIds() : 0;
+        connectivitySize += polys ? polys->GetNumberOfConnectivityIds() : 0;
+        connectivitySize += strips ? strips->GetNumberOfConnectivityIds() : 0;
+        FastUnstructuredDataSCLT(connectivitySize, polyData, processedCellData, outPD);
+      }
       return 1;
     }
   } // fast path
@@ -545,15 +646,14 @@ int vtkCellDataToPointData::RequestDataForUnstructuredData(
   if (this->ContributingCellOption != vtkCellDataToPointData::Patch)
   {
     num = vtkSmartPointer<vtkUnsignedIntArray>::New();
-    num->SetNumberOfComponents(1);
-    num->SetNumberOfTuples(npoints);
-    std::fill_n(num->GetPointer(0), npoints, 0u);
+    num->SetNumberOfValues(numberOfPoints);
+    num->FillValue(0);
     if (this->ContributingCellOption == vtkCellDataToPointData::DataSetMax)
     {
-      int maxDimension = src->IsA("vtkPolyData") == 1 ? 2 : 3;
-      for (vtkIdType i = 0; i < src->GetNumberOfCells(); i++)
+      int maxDimension = input->IsA("vtkPolyData") == 1 ? 2 : 3;
+      for (vtkIdType i = 0; i < numberOfCells; i++)
       {
-        int dim = vtkCellTypes::GetDimension(src->GetCellType(i));
+        int dim = vtkCellTypes::GetDimension(input->GetCellType(i));
         if (dim > highestCellDimension)
         {
           highestCellDimension = dim;
@@ -565,11 +665,11 @@ int vtkCellDataToPointData::RequestDataForUnstructuredData(
       }
     }
     vtkNew<vtkIdList> pids;
-    for (vtkIdType cid = 0; cid < ncells; ++cid)
+    for (vtkIdType cid = 0; cid < numberOfCells; ++cid)
     {
-      if (src->GetCell(cid)->GetCellDimension() >= highestCellDimension)
+      if (input->GetCell(cid)->GetCellDimension() >= highestCellDimension)
       {
-        src->GetCellPoints(cid, pids);
+        input->GetCellPoints(cid, pids);
         for (vtkIdType i = 0, I = pids->GetNumberOfIds(); i < I; ++i)
         {
           vtkIdType const pid = pids->GetId(i);
@@ -581,38 +681,37 @@ int vtkCellDataToPointData::RequestDataForUnstructuredData(
 
   const auto nfields = processedCellData->GetNumberOfArrays();
   int fid = 0;
-  auto f = [this, &fid, nfields, npoints, src, num, ncells, highestCellDimension](
-             vtkAbstractArray* aa_srcarray, vtkAbstractArray* aa_dstarray) {
+  auto f = [this, &fid, nfields, numberOfPoints, input, num, numberOfCells, highestCellDimension](
+             vtkAbstractArray* aa_srcarray, vtkAbstractArray* aa_dstarray)
+  {
     // update progress and check for an abort request.
     this->UpdateProgress((fid + 1.0) / nfields);
     ++fid;
 
-    if (this->GetAbortExecute())
-    {
-      return;
-    }
-
     vtkDataArray* const srcarray = vtkDataArray::FastDownCast(aa_srcarray);
     vtkDataArray* const dstarray = vtkDataArray::FastDownCast(aa_dstarray);
     if (srcarray && dstarray)
     {
-      dstarray->SetNumberOfTuples(npoints);
+      dstarray->SetNumberOfTuples(numberOfPoints);
       vtkIdType const ncomps = srcarray->GetNumberOfComponents();
 
       Spread worker;
       using Dispatcher = vtkArrayDispatch::Dispatch2SameValueType;
-      if (!Dispatcher::Execute(srcarray, dstarray, worker, src, num, ncells, npoints, ncomps,
-            highestCellDimension, this->ContributingCellOption))
+      if (!Dispatcher::Execute(srcarray, dstarray, worker, input, num, numberOfCells,
+            numberOfPoints, ncomps, highestCellDimension, this->ContributingCellOption, this))
       { // fallback for unknown arrays:
-        worker(srcarray, dstarray, src, num, ncells, npoints, ncomps, highestCellDimension,
-          this->ContributingCellOption);
+        worker(srcarray, dstarray, input, num, numberOfCells, numberOfPoints, ncomps,
+          highestCellDimension, this->ContributingCellOption, this);
       }
     }
   };
 
-  if (processedCellData != nullptr && dst->GetPointData() != nullptr)
+  // Cell field list constructed from the filtered cell data array
+  vtkDataSetAttributes::FieldList cfl(1);
+  cfl.InitializeFieldList(processedCellData);
+  if (processedCellData != nullptr && outPD != nullptr)
   {
-    cfl.TransformData(0, processedCellData, dst->GetPointData(), f);
+    cfl.TransformData(0, processedCellData, outPD, f);
   }
 
   return 1; // slow path
@@ -624,44 +723,44 @@ int vtkCellDataToPointData::InterpolatePointData(vtkDataSet* input, vtkDataSet*
   vtkNew<vtkIdList> cellIds;
   cellIds->Allocate(VTK_MAX_CELLS_PER_POINT);
 
-  vtkIdType numPts = input->GetNumberOfPoints();
+  const vtkIdType numberOfPoints = input->GetNumberOfPoints();
 
-  vtkCellData* inputInCD = input->GetCellData();
-  vtkCellData* inCD;
+  vtkCellData* inCD = input->GetCellData();
   vtkPointData* outPD = output->GetPointData();
 
+  // Copy all existing cell fields into a temporary cell data array,
+  // unless the SelectCellDataArrays option is active.
+  vtkNew<vtkCellData> processedCellData;
   if (!this->ProcessAllArrays)
   {
-    inCD = vtkCellData::New();
-
     for (const auto& name : this->Implementation->CellDataArrays)
     {
-      vtkAbstractArray* arr = inputInCD->GetAbstractArray(name.c_str());
-      if (arr == nullptr)
+      vtkAbstractArray* array = inCD->GetAbstractArray(name.c_str());
+      if (!array)
       {
         vtkWarningMacro("cell data array name not found.");
         continue;
       }
-      inCD->AddArray(arr);
+      processedCellData->AddArray(array);
     }
   }
   else
   {
-    inCD = inputInCD;
+    processedCellData->ShallowCopy(inCD);
   }
 
-  outPD->InterpolateAllocate(inCD, numPts);
+  outPD->InterpolateAllocate(processedCellData, numberOfPoints);
 
   double weights[VTK_MAX_CELLS_PER_POINT];
 
-  int abort = 0;
-  vtkIdType progressInterval = numPts / 20 + 1;
-  for (vtkIdType ptId = 0; ptId < numPts && !abort; ptId++)
+  bool abort = false;
+  vtkIdType progressInterval = numberOfPoints / 20 + 1;
+  for (vtkIdType ptId = 0; ptId < numberOfPoints && !abort; ptId++)
   {
     if (!(ptId % progressInterval))
     {
-      this->UpdateProgress(static_cast<double>(ptId) / numPts);
-      abort = GetAbortExecute();
+      this->UpdateProgress(static_cast<double>(ptId) / numberOfPoints);
+      abort = this->CheckAbort();
     }
 
     input->GetPointCells(ptId, cellIds);
@@ -674,7 +773,7 @@ int vtkCellDataToPointData::InterpolatePointData(vtkDataSet* input, vtkDataSet*
       {
         weights[cellId] = weight;
       }
-      outPD->InterpolatePoint(inCD, ptId, cellIds, weights);
+      outPD->InterpolatePoint(processedCellData, ptId, cellIds, weights);
     }
     else
     {
@@ -682,10 +781,6 @@ int vtkCellDataToPointData::InterpolatePointData(vtkDataSet* input, vtkDataSet*
     }
   }
 
-  if (!this->ProcessAllArrays)
-  {
-    inCD->Delete();
-  }
-
   return 1;
 }
+VTK_ABI_NAMESPACE_END
