@@ -1,17 +1,5 @@
-/*=========================================================================
-
-Program:   Visualization Toolkit
-Module:    vtkHyperTreeGridSource.cxx
-
-Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-All rights reserved.
-See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-This software is distributed WITHOUT ANY WARRANTY; without even
-the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkHyperTreeGridSource.h"
 
 #include "vtkBitArray.h"
@@ -33,6 +21,7 @@ PURPOSE.  See the above copyright notice for more information.
 #include <cassert>
 #include <sstream>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkHyperTreeGridSource);
 vtkCxxSetObjectMacro(vtkHyperTreeGridSource, DescriptorBits, vtkBitArray);
 vtkCxxSetObjectMacro(vtkHyperTreeGridSource, MaskBits, vtkBitArray);
@@ -360,6 +349,7 @@ int vtkHyperTreeGridSource::RequestInformation(
   extent[4] = 0;
   extent[5] = this->Dimensions[2] - 1;
   outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), extent, 6);
+  outInfo->Set(CAN_HANDLE_PIECE_REQUEST(), 1);
 
   return 1;
 }
@@ -377,12 +367,16 @@ int vtkHyperTreeGridSource::RequestData(
     return 0;
   }
 
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
+  this->Piece = outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER());
+  this->NumPieces = outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES());
+
   output->Initialize();
-  if (this->UseMask)
-  {
-    vtkNew<vtkBitArray> mask;
-    output->SetMask(mask);
-  }
+
+  // A mask is required when using UseMask or when assigning trees to pieces,
+  // so we create it everytime.
+  vtkNew<vtkBitArray> mask;
+  output->SetMask(mask);
 
   vtkCellData* outData = output->GetCellData();
 
@@ -401,10 +395,12 @@ int vtkHyperTreeGridSource::RequestData(
 
     if (!this->DescriptorBits && !this->InitializeFromStringDescriptor())
     {
+      vtkErrorMacro(<< "Could not initialize string descriptor.");
       return 0;
     }
     else if (this->DescriptorBits && !this->InitializeFromBitsDescriptor())
     {
+      vtkErrorMacro(<< "Could not initialize bits descriptor.");
       return 0;
     }
   } // if this->UseDescriptor
@@ -618,6 +614,9 @@ int vtkHyperTreeGridSource::ProcessTrees(vtkHyperTreeGrid*, vtkDataObject* outpu
     return 0;
   }
 
+  // Reset process counter
+  this->CurrentTreeProcess = 0;
+
   // Iterate over all hyper trees
   vtkIdType nbTrees;
   if (this->LevelZeroMaterialIndex)
@@ -629,6 +628,7 @@ int vtkHyperTreeGridSource::ProcessTrees(vtkHyperTreeGrid*, vtkDataObject* outpu
     nbTrees = output->GetMaxNumberOfTrees();
   }
   vtkNew<vtkHyperTreeGridNonOrientedCursor> cursor;
+  vtkIdType offset = 0;
   for (vtkIdType itree = 0; itree < nbTrees; ++itree)
   {
     vtkIdType index = itree;
@@ -649,7 +649,21 @@ int vtkHyperTreeGridSource::ProcessTrees(vtkHyperTreeGrid*, vtkDataObject* outpu
 
     if (this->UseDescriptor)
     {
-      this->InitTreeFromDescriptor(output, cursor, index, idx);
+      if (!this->DescriptorBits)
+      {
+        char currentChar = this->LevelDescriptors[0][index + offset];
+        if (currentChar != 'R' && currentChar != '.')
+        {
+          this->CurrentTreeProcess = currentChar - '0';
+          offset++;
+          if (this->CurrentTreeProcess < 0 || this->CurrentTreeProcess > 9)
+          {
+            vtkErrorMacro("Unexpected level " << CurrentTreeProcess);
+            return 0;
+          }
+        }
+      }
+      this->InitTreeFromDescriptor(output, cursor, index, idx, offset);
     }
     else
     {
@@ -672,13 +686,13 @@ int vtkHyperTreeGridSource::ProcessTrees(vtkHyperTreeGrid*, vtkDataObject* outpu
 }
 
 //------------------------------------------------------------------------------
-void vtkHyperTreeGridSource::InitTreeFromDescriptor(
-  vtkHyperTreeGrid* output, vtkHyperTreeGridNonOrientedCursor* cursor, int treeIdx, int idx[3])
+void vtkHyperTreeGridSource::InitTreeFromDescriptor(vtkHyperTreeGrid* output,
+  vtkHyperTreeGridNonOrientedCursor* cursor, int treeIdx, int idx[3], int offset)
 {
   // Subdivide using descriptor
   if (!this->DescriptorBits)
   {
-    this->SubdivideFromStringDescriptor(output, cursor, 0, treeIdx, 0, idx, 0);
+    this->SubdivideFromStringDescriptor(output, cursor, 0, treeIdx, 0, idx, 0, offset);
   }
   else
   {
@@ -688,14 +702,19 @@ void vtkHyperTreeGridSource::InitTreeFromDescriptor(
 //------------------------------------------------------------------------------
 int vtkHyperTreeGridSource::InitializeFromStringDescriptor()
 {
-  size_t descLen = strlen(this->Descriptor);
+  std::string descNoProcess = this->Descriptor;
+  for (char c = '0'; c <= '9'; c++)
+  {
+    descNoProcess.erase(
+      std::remove(descNoProcess.begin(), descNoProcess.end(), c), descNoProcess.end());
+  }
 
   // Verify that grid and material specifications are consistent
-  if (this->UseMask && strlen(this->Mask) != descLen)
+  if (this->UseMask && strlen(this->Mask) != descNoProcess.size())
   {
     vtkErrorMacro(<< "Material mask is used but has length " << strlen(this->Mask)
-                  << " != " << descLen << " which is the length of the grid descriptor.");
-
+                  << " != " << descNoProcess.size()
+                  << " which is the length of the grid descriptor, omitting process qualifiers.");
     return 0;
   }
 
@@ -713,6 +732,7 @@ int vtkHyperTreeGridSource::InitializeFromStringDescriptor()
   unsigned int nRefined = 0;
   unsigned int nLeaves = 0;
   unsigned int nNextLevel = nTotal;
+  unsigned int maskCounter = 0;
   bool rootLevel = true;
   std::ostringstream descriptor;
   std::ostringstream mask;
@@ -721,10 +741,16 @@ int vtkHyperTreeGridSource::InitializeFromStringDescriptor()
   this->LevelDescriptors.clear();
   this->LevelMasks.clear();
 
-  for (size_t i = 0; i < descLen; ++i)
+  // Iterate through descriptor strings
+  for (size_t i = 0; i < strlen(this->Descriptor); ++i)
   {
     char c = this->Descriptor[i];
-    char m = this->UseMask ? this->Mask[i] : 0;
+    char m = 0;
+    if (!isdigit(c) && this->UseMask)
+    {
+      // Only read mask value when the current descriptor character is not a process number
+      m = this->Mask[maskCounter++];
+    }
     switch (c)
     {
       case ' ':
@@ -750,31 +776,14 @@ int vtkHyperTreeGridSource::InitializeFromStringDescriptor()
         this->LevelDescriptors.push_back(descriptor.str());
         this->LevelMasks.push_back(mask.str());
 
-        // Check whether cursor is still at rool level
-        if (rootLevel)
+        if (!this->IsLevelDescriptorConsistent(
+              rootLevel, nRefined, nLeaves, nTotal, nNextLevel, descriptor))
         {
-          rootLevel = false;
+          return 0;
+        }
 
-          // Verify that total number of root cells is consistent with descriptor
-          if (nRefined + nLeaves != nTotal)
-          {
-            vtkErrorMacro(<< "String " << this->Descriptor << " describes " << nRefined + nLeaves
-                          << " root cells != " << nTotal);
-            return 0;
-          }
-        } // if (rootLevel)
-        else
-        {
-          // Verify that level descriptor cardinality matches expected value
-          if (descriptor.str().size() != nNextLevel)
-          {
-            vtkErrorMacro(<< "String level descriptor " << descriptor.str().c_str()
-                          << " has cardinality " << descriptor.str().size()
-                          << " which is not expected value of " << nNextLevel);
-
-            return 0;
-          }
-        } // else
+        // Changing level means we're not are root level
+        rootLevel = false;
 
         // Predict next level descriptor cardinality
         nNextLevel = nRefined * this->BlockSize;
@@ -786,12 +795,11 @@ int vtkHyperTreeGridSource::InitializeFromStringDescriptor()
         nLeaves = 0;
         break; // case '|'
 
-      case '1':
       case 'R':
         //  Refined cell, verify mask consistency if needed
         if (this->UseMask && m == '0')
         {
-          vtkErrorMacro(<< "A refined branch must contain material.");
+          vtkErrorMacro(<< "A refined branch cannot be masked.");
           return 0;
         }
         // Refined cell, update branch counter
@@ -805,7 +813,6 @@ int vtkHyperTreeGridSource::InitializeFromStringDescriptor()
         }
         break; // case 'R'
 
-      case '0':
       case '.':
         // Leaf cell, update leaf counter
         ++nLeaves;
@@ -817,21 +824,49 @@ int vtkHyperTreeGridSource::InitializeFromStringDescriptor()
           mask << m;
         }
         break; // case '.'
+      case '0':
+      case '1':
+      case '2':
+      case '3':
+      case '4':
+      case '5':
+      case '6':
+      case '7':
+      case '8':
+      case '9':
+      {
+        // Process-specific tree. Can only be used at root level
+        if (!rootLevel)
+        {
+          vtkErrorMacro(
+            << "It is only possible to use digits to bind trees to parallel pieces at root level");
+          return 1;
+        }
+        char piece = c - '0';
+        if (piece > this->NumPieces)
+        {
+          vtkErrorMacro(<< "Can not assign tree to piece " << piece
+                        << ". Available number of pieces: " << this->NumPieces);
+          return 1;
+        }
 
+        // Simply append into the descriptor
+        descriptor << c;
+
+        break; // case 'digit'
+      }
       default:
         vtkErrorMacro(<< "Unrecognized character: " << c << " at pos " << i << " in descriptor "
                       << this->Descriptor);
 
-        return 0; // default
-    }             // switch(c)
-  }               // c
+        return 1;
+    } // switch(c)
+  }   // char loop
 
   // Verify and append last level string
-  if (descriptor.str().size() != nNextLevel)
+  if (!this->IsLevelDescriptorConsistent(
+        rootLevel, nRefined, nLeaves, nTotal, nNextLevel, descriptor))
   {
-    vtkErrorMacro(<< "String level descriptor " << descriptor.str().c_str() << " has cardinality "
-                  << descriptor.str().size() << " which is not expected value of " << nNextLevel);
-
     return 0;
   }
 
@@ -850,7 +885,7 @@ int vtkHyperTreeGridSource::InitializeFromStringDescriptor()
     nNextLevel = nRefined * this->BlockSize;
     if (nRefined > 0)
     {
-      this->LevelDescriptors.emplace_back(std::string(nNextLevel, '.'));
+      this->LevelDescriptors.emplace_back(nNextLevel, '.');
     }
   }
 
@@ -872,16 +907,45 @@ int vtkHyperTreeGridSource::InitializeFromStringDescriptor()
   return 1;
 }
 
+bool vtkHyperTreeGridSource::IsLevelDescriptorConsistent(bool isRootLevel, unsigned int nRefined,
+  unsigned int nLeaves, unsigned int nTotal, unsigned int nNextLevel,
+  const std::ostringstream& descriptor)
+{
+  // Check whether cursor is still at rool level
+  if (isRootLevel)
+  {
+    // Verify that total number of root cells is consistent with descriptor
+    if (nRefined + nLeaves != nTotal)
+    {
+      vtkErrorMacro(<< "String " << this->Descriptor << " describes " << nRefined + nLeaves
+                    << " root cells != " << nTotal);
+      return false;
+    }
+  } // if (rootLevel)
+  else
+  {
+    // Verify that level descriptor cardinality matches expected value
+    if (descriptor.str().size() != nNextLevel)
+    {
+      vtkErrorMacro(<< "String level descriptor " << descriptor.str() << " has cardinality "
+                    << descriptor.str().size() << " which is not expected value of " << nNextLevel);
+      return false;
+    }
+  }
+
+  return true;
+}
+
 //------------------------------------------------------------------------------
 void vtkHyperTreeGridSource::SubdivideFromStringDescriptor(vtkHyperTreeGrid* output,
   vtkHyperTreeGridNonOrientedCursor* cursor, unsigned int level, int treeIdx, int childIdx,
-  int idx[3], int parentPos)
+  int idx[3], int parentPos, int offset)
 {
   // Get handle on point data
   vtkCellData* outData = output->GetCellData();
 
   // Calculate pointer into level descriptor string
-  unsigned int pointer = level ? childIdx + parentPos * this->BlockSize : treeIdx;
+  unsigned int pointer = level ? childIdx + parentPos * this->BlockSize : treeIdx + offset;
 
   // Calculate the node global index
   vtkIdType id = this->LevelBitsIndexCnt[level];
@@ -898,8 +962,9 @@ void vtkHyperTreeGridSource::SubdivideFromStringDescriptor(vtkHyperTreeGrid* out
     outData->GetArray("Intercepts")->InsertTuple3(id, v, 0., 3.);
   }
 
-  // Initialize global index of tree
+  // Initialize global index of tree and mask state
   cursor->SetGlobalIndexFromLocal(id);
+  cursor->SetMask(false);
 
   // Subdivide further or stop recursion with terminal leaf
   if (level + 1 < this->MaxDepth &&
@@ -912,12 +977,7 @@ void vtkHyperTreeGridSource::SubdivideFromStringDescriptor(vtkHyperTreeGrid* out
     //      set value by tree with SetGlobalIndexStart only once
     //    if explicit
     //      set value by cell with SetGlobalIndexFromLocal
-    // 2) if use mask
-    //    set mask to false
-    if (this->UseMask)
-    {
-      cursor->SetMask(false);
-    }
+    // 2) set mask to false
 
     // Subdivide hyper tree grid leaf
     cursor->SubdivideLeaf();
@@ -983,8 +1043,8 @@ void vtkHyperTreeGridSource::SubdivideFromStringDescriptor(vtkHyperTreeGrid* out
           cursor->ToChild(newChildIdx);
 
           // Recurse
-          this->SubdivideFromStringDescriptor(
-            output, cursor, level + 1, treeIdx, newChildIdx, newIdx, this->LevelCounters.at(level));
+          this->SubdivideFromStringDescriptor(output, cursor, level + 1, treeIdx, newChildIdx,
+            newIdx, this->LevelCounters.at(level), 0);
 
           // Reset cursor to parent
           cursor->ToParent();
@@ -1001,9 +1061,15 @@ void vtkHyperTreeGridSource::SubdivideFromStringDescriptor(vtkHyperTreeGrid* out
   else if (this->UseMask)
   {
     // Blank leaf if needed
-    bool masked = this->LevelMasks.at(level).at(pointer) == '0';
+    bool masked = this->LevelMasks.at(level).at(pointer - offset) == '0';
     output->GetMask()->InsertTuple1(id, masked);
   } // else if
+
+  // Process selection for root trees: mask the entire tree if it's not selected for this process
+  if (level == 0 && this->CurrentTreeProcess != this->Piece)
+  {
+    cursor->SetMask(true);
+  }
 }
 
 //------------------------------------------------------------------------------
@@ -1069,7 +1135,7 @@ int vtkHyperTreeGridSource::InitializeFromBitsDescriptor()
     vtkErrorMacro(<< "Level descriptor " << nCurrentLevel << " has cardinality "
                   << nCurrentLevelCount << " which is not expected value of " << nNextLevel);
 
-    return 0;
+    return 1;
   }
 
   ++nCurrentLevel;
@@ -1535,3 +1601,4 @@ vtkBitArray* vtkHyperTreeGridSource::ConvertMaskStringToBitArray(const std::stri
 {
   return ConvertDescriptorStringToBitArray(str);
 }
+VTK_ABI_NAMESPACE_END
