@@ -1,22 +1,13 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkGLTFImporter.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkGLTFImporter.h"
 
 #include "vtkActor.h"
+#include "vtkActorCollection.h"
 #include "vtkCamera.h"
+#include "vtkCollection.h"
+#include "vtkDataAssembly.h"
 #include "vtkDoubleArray.h"
 #include "vtkEventForwarderCommand.h"
 #include "vtkFloatArray.h"
@@ -27,6 +18,7 @@
 #include "vtkImageResize.h"
 #include "vtkInformation.h"
 #include "vtkLight.h"
+#include "vtkLightCollection.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 #include "vtkPolyDataMapper.h"
@@ -43,8 +35,10 @@
 
 #include <algorithm>
 #include <array>
+#include <set>
 #include <stack>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkGLTFImporter);
 
 namespace
@@ -79,7 +73,7 @@ vtkSmartPointer<vtkCamera> GLTFCameraToVTKCamera(const vtkGLTFDocumentLoader::Ca
 //------------------------------------------------------------------------------
 vtkSmartPointer<vtkTexture> CreateVTKTextureFromGLTFTexture(
   std::shared_ptr<vtkGLTFDocumentLoader::Model> model, int textureIndex,
-  std::map<int, vtkSmartPointer<vtkTexture>>& existingTextures)
+  std::map<int, vtkSmartPointer<vtkTexture>>& existingTextures, vtkGLTFImporter* parent)
 {
 
   if (existingTextures.count(textureIndex))
@@ -90,11 +84,16 @@ vtkSmartPointer<vtkTexture> CreateVTKTextureFromGLTFTexture(
   const vtkGLTFDocumentLoader::Texture& glTFTex = model->Textures[textureIndex];
   if (glTFTex.Source < 0 || glTFTex.Source >= static_cast<int>(model->Images.size()))
   {
-    vtkErrorWithObjectMacro(nullptr, "Image not found");
+    vtkErrorWithObjectMacro(parent, "Image not found");
     return nullptr;
   }
 
   const vtkGLTFDocumentLoader::Image& image = model->Images[glTFTex.Source];
+  if (image.ImageData == nullptr)
+  {
+    vtkErrorWithObjectMacro(parent, "Image mimeType not supported");
+    return nullptr;
+  }
 
   vtkNew<vtkTexture> texture;
   texture->SetColorModeToDirectScalars();
@@ -159,14 +158,25 @@ vtkSmartPointer<vtkTexture> CreateVTKTextureFromGLTFTexture(
 }
 
 //------------------------------------------------------------------------------
-bool MaterialHasMultipleUVs(const vtkGLTFDocumentLoader::Material& material)
+bool MaterialHasMultipleUVs(
+  const vtkGLTFDocumentLoader::Material& material, vtkGLTFImporter* parent)
 {
-  int firstUV = material.PbrMetallicRoughness.BaseColorTexture.TexCoord;
-  return (material.EmissiveTexture.Index >= 0 && material.EmissiveTexture.TexCoord != firstUV) ||
-    (material.NormalTexture.Index >= 0 && material.NormalTexture.TexCoord != firstUV) ||
-    (material.OcclusionTexture.Index >= 0 && material.OcclusionTexture.TexCoord != firstUV) ||
-    (material.PbrMetallicRoughness.MetallicRoughnessTexture.Index >= 0 &&
-      material.PbrMetallicRoughness.MetallicRoughnessTexture.TexCoord != firstUV);
+  std::set<int> uv_sets;
+  for (const vtkGLTFDocumentLoader::TextureInfo& info :
+    { material.PbrMetallicRoughness.BaseColorTexture, material.EmissiveTexture,
+      material.NormalTexture, material.OcclusionTexture,
+      material.PbrMetallicRoughness.MetallicRoughnessTexture })
+  {
+    if (info.Index >= 0)
+      uv_sets.insert(info.TexCoord);
+  }
+  if (uv_sets.find(-1) != uv_sets.end())
+  {
+    vtkErrorWithObjectMacro(parent,
+      "A material defined a texture index without "
+      "defining a texture coordinate set.");
+  }
+  return uv_sets.size() > 1;
 }
 
 //------------------------------------------------------------------------------
@@ -185,17 +195,17 @@ bool PrimitiveNeedsTangents(const std::shared_ptr<vtkGLTFDocumentLoader::Model>
 }
 
 //------------------------------------------------------------------------------
-void ApplyGLTFMaterialToVTKActor(std::shared_ptr<vtkGLTFDocumentLoader::Model> model,
+bool ApplyGLTFMaterialToVTKActor(std::shared_ptr<vtkGLTFDocumentLoader::Model> model,
   vtkGLTFDocumentLoader::Primitive& primitive, vtkSmartPointer<vtkActor> actor,
-  std::map<int, vtkSmartPointer<vtkTexture>>& existingTextures)
+  std::map<int, vtkSmartPointer<vtkTexture>>& existingTextures, vtkGLTFImporter* parent)
 {
   vtkGLTFDocumentLoader::Material& material = model->Materials[primitive.Material];
 
-  bool hasMultipleUVs = MaterialHasMultipleUVs(material);
+  bool hasMultipleUVs = MaterialHasMultipleUVs(material, parent);
   if (hasMultipleUVs)
   {
     vtkWarningWithObjectMacro(
-      nullptr, "Using multiple texture coordinates for the same model is not supported.");
+      parent, "Using multiple texture coordinates for the same model is not supported.");
   }
   auto property = actor->GetProperty();
   property->SetInterpolationToPBR();
@@ -232,85 +242,104 @@ void ApplyGLTFMaterialToVTKActor(std::shared_ptr<vtkGLTFDocumentLoader::Model> m
   {
     // set albedo texture
     vtkSmartPointer<vtkTexture> baseColorTex;
-    baseColorTex = CreateVTKTextureFromGLTFTexture(model, texIndex, existingTextures);
+    baseColorTex = CreateVTKTextureFromGLTFTexture(model, texIndex, existingTextures, parent);
+    if (!baseColorTex)
+    {
+      return false;
+    }
     baseColorTex->UseSRGBColorSpaceOn();
     property->SetBaseColorTexture(baseColorTex);
+  }
 
-    // merge ambient occlusion and metallic/roughness, then set material texture
-    int pbrTexIndex = material.PbrMetallicRoughness.MetallicRoughnessTexture.Index;
-    if (pbrTexIndex >= 0 && pbrTexIndex < static_cast<int>(model->Textures.size()))
+  // merge ambient occlusion and metallic/roughness, then set material texture
+  int pbrTexIndex = material.PbrMetallicRoughness.MetallicRoughnessTexture.Index;
+  if (pbrTexIndex >= 0 && pbrTexIndex < static_cast<int>(model->Textures.size()))
+  {
+    const vtkGLTFDocumentLoader::Texture& pbrTexture = model->Textures[pbrTexIndex];
+    if (pbrTexture.Source >= 0 && pbrTexture.Source < static_cast<int>(model->Images.size()))
     {
-      const vtkGLTFDocumentLoader::Texture& pbrTexture = model->Textures[pbrTexIndex];
-      if (pbrTexture.Source >= 0 && pbrTexture.Source < static_cast<int>(model->Images.size()))
+      const vtkGLTFDocumentLoader::Image& pbrImage = model->Images[pbrTexture.Source];
+      // While glTF 2.0 uses two different textures for Ambient Occlusion and Metallic/Roughness
+      // values, VTK only uses one, so we merge both textures into one.
+      // If an Ambient Occlusion texture is present, we merge its first channel into the
+      // metallic/roughness texture (AO is r, Roughness g and Metallic b) If no Ambient
+      // Occlusion texture is present, we need to fill the metallic/roughness texture's first
+      // channel with 255
+      int aoTexIndex = material.OcclusionTexture.Index;
+      if (!hasMultipleUVs && aoTexIndex >= 0 &&
+        aoTexIndex < static_cast<int>(model->Textures.size()))
       {
-        const vtkGLTFDocumentLoader::Image& pbrImage = model->Images[pbrTexture.Source];
-        // While glTF 2.0 uses two different textures for Ambient Occlusion and Metallic/Roughness
-        // values, VTK only uses one, so we merge both textures into one.
-        // If an Ambient Occlusion texture is present, we merge its first channel into the
-        // metallic/roughness texture (AO is r, Roughness g and Metallic b) If no Ambient
-        // Occlusion texture is present, we need to fill the metallic/roughness texture's first
-        // channel with 255
-        int aoTexIndex = material.OcclusionTexture.Index;
-        if (!hasMultipleUVs && aoTexIndex >= 0 &&
-          aoTexIndex < static_cast<int>(model->Textures.size()))
+        actor->GetProperty()->SetOcclusionStrength(material.OcclusionTextureStrength);
+        const vtkGLTFDocumentLoader::Texture& aoTexture = model->Textures[aoTexIndex];
+        const vtkGLTFDocumentLoader::Image& aoImage = model->Images[aoTexture.Source];
+        vtkNew<vtkImageExtractComponents> redAO;
+        // If sizes are different, resize the AO texture to the R/M texture's size
+        std::array<vtkIdType, 3> aoSize = { { 0 } };
+        std::array<vtkIdType, 3> pbrSize = { { 0 } };
+        aoImage.ImageData->GetDimensions(aoSize.data());
+        pbrImage.ImageData->GetDimensions(pbrSize.data());
+        // compare dimensions
+        if (aoSize != pbrSize)
         {
-          actor->GetProperty()->SetOcclusionStrength(material.OcclusionTextureStrength);
-          const vtkGLTFDocumentLoader::Texture& aoTexture = model->Textures[aoTexIndex];
-          const vtkGLTFDocumentLoader::Image& aoImage = model->Images[aoTexture.Source];
-          vtkNew<vtkImageExtractComponents> redAO;
-          // If sizes are different, resize the AO texture to the R/M texture's size
-          std::array<vtkIdType, 3> aoSize = { { 0 } };
-          std::array<vtkIdType, 3> pbrSize = { { 0 } };
-          aoImage.ImageData->GetDimensions(aoSize.data());
-          pbrImage.ImageData->GetDimensions(pbrSize.data());
-          // compare dimensions
-          if (aoSize != pbrSize)
-          {
-            vtkNew<vtkImageResize> resize;
-            resize->SetInputData(aoImage.ImageData);
-            resize->SetOutputDimensions(pbrSize[0], pbrSize[1], pbrSize[2]);
-            resize->Update();
-            redAO->SetInputConnection(resize->GetOutputPort(0));
-          }
-          else
-          {
-            redAO->SetInputData(aoImage.ImageData);
-          }
-          redAO->SetComponents(0);
-          vtkNew<vtkImageExtractComponents> gbPbr;
-          gbPbr->SetInputData(pbrImage.ImageData);
-          gbPbr->SetComponents(1, 2);
-          vtkNew<vtkImageAppendComponents> append;
-          append->AddInputConnection(redAO->GetOutputPort());
-          append->AddInputConnection(gbPbr->GetOutputPort());
-          append->SetOutput(pbrImage.ImageData);
-          append->Update();
+          vtkNew<vtkImageResize> resize;
+          resize->SetInputData(aoImage.ImageData);
+          resize->SetOutputDimensions(pbrSize[0], pbrSize[1], pbrSize[2]);
+          resize->Update();
+          redAO->SetInputConnection(resize->GetOutputPort(0));
         }
         else
         {
-          pbrImage.ImageData->GetPointData()->GetScalars()->FillComponent(0, 255);
+          redAO->SetInputData(aoImage.ImageData);
         }
-        auto materialTex = CreateVTKTextureFromGLTFTexture(model, pbrTexIndex, existingTextures);
-        property->SetORMTexture(materialTex);
+        redAO->SetComponents(0);
+        vtkNew<vtkImageExtractComponents> gbPbr;
+        gbPbr->SetInputData(pbrImage.ImageData);
+        gbPbr->SetComponents(1, 2);
+        vtkNew<vtkImageAppendComponents> append;
+        append->AddInputConnection(redAO->GetOutputPort());
+        append->AddInputConnection(gbPbr->GetOutputPort());
+        append->SetOutput(pbrImage.ImageData);
+        append->Update();
       }
+      else
+      {
+        pbrImage.ImageData->GetPointData()->GetScalars()->FillComponent(0, 255);
+      }
+      auto materialTex =
+        CreateVTKTextureFromGLTFTexture(model, pbrTexIndex, existingTextures, parent);
+      if (!materialTex)
+      {
+        return false;
+      }
+      property->SetORMTexture(materialTex);
     }
+  }
 
-    // Set emissive texture
-    int emissiveTexIndex = material.EmissiveTexture.Index;
-    if (emissiveTexIndex >= 0 && emissiveTexIndex < static_cast<int>(model->Textures.size()))
+  // Set emissive texture
+  int emissiveTexIndex = material.EmissiveTexture.Index;
+  if (emissiveTexIndex >= 0 && emissiveTexIndex < static_cast<int>(model->Textures.size()))
+  {
+    auto emissiveTex =
+      CreateVTKTextureFromGLTFTexture(model, emissiveTexIndex, existingTextures, parent);
+    if (!emissiveTex)
     {
-      auto emissiveTex = CreateVTKTextureFromGLTFTexture(model, emissiveTexIndex, existingTextures);
-      emissiveTex->UseSRGBColorSpaceOn();
-      property->SetEmissiveTexture(emissiveTex);
+      return false;
     }
-    // Set normal map
-    int normalMapIndex = material.NormalTexture.Index;
-    if (normalMapIndex >= 0 && normalMapIndex < static_cast<int>(model->Textures.size()))
+    emissiveTex->UseSRGBColorSpaceOn();
+    property->SetEmissiveTexture(emissiveTex);
+  }
+  // Set normal map
+  int normalMapIndex = material.NormalTexture.Index;
+  if (normalMapIndex >= 0 && normalMapIndex < static_cast<int>(model->Textures.size()))
+  {
+    actor->GetProperty()->SetNormalScale(material.NormalTextureScale);
+    auto normalTex =
+      CreateVTKTextureFromGLTFTexture(model, normalMapIndex, existingTextures, parent);
+    if (!normalTex)
     {
-      actor->GetProperty()->SetNormalScale(material.NormalTextureScale);
-      auto normalTex = CreateVTKTextureFromGLTFTexture(model, normalMapIndex, existingTextures);
-      property->SetNormalTexture(normalTex);
+      return false;
     }
+    property->SetNormalTexture(normalTex);
   }
 
   // extension KHR_materials_unlit
@@ -325,7 +354,8 @@ void ApplyGLTFMaterialToVTKActor(std::shared_ptr<vtkGLTFDocumentLoader::Model> m
     b = pow(b, 1.f / 2.2f);
     actor->GetProperty()->SetColor(r, g, b);
   }
-};
+  return true;
+}
 
 //------------------------------------------------------------------------------
 void ApplyTransformToCamera(vtkSmartPointer<vtkCamera> cam, vtkSmartPointer<vtkMatrix4x4> transform)
@@ -363,6 +393,12 @@ vtkGLTFImporter::~vtkGLTFImporter()
   this->SetFileName(nullptr);
 }
 
+//------------------------------------------------------------------------------
+void vtkGLTFImporter::InitializeLoader()
+{
+  this->Loader = vtkSmartPointer<vtkGLTFDocumentLoader>::New();
+}
+
 //------------------------------------------------------------------------------
 int vtkGLTFImporter::ImportBegin()
 {
@@ -375,11 +411,14 @@ int vtkGLTFImporter::ImportBegin()
 
   this->Textures.clear();
 
-  this->Loader = vtkSmartPointer<vtkGLTFDocumentLoader>::New();
+  this->InitializeLoader();
+  this->SceneHierarchy.TakeReference(vtkDataAssembly::New());
 
   vtkNew<vtkEventForwarderCommand> forwarder;
   forwarder->SetTarget(this);
   this->Loader->AddObserver(vtkCommand::ProgressEvent, forwarder);
+  this->Loader->AddObserver(vtkCommand::WarningEvent, forwarder);
+  this->Loader->AddObserver(vtkCommand::ErrorEvent, forwarder);
 
   // Check extension
   std::vector<char> glbBuffer;
@@ -419,34 +458,59 @@ int vtkGLTFImporter::ImportBegin()
 void vtkGLTFImporter::ImportActors(vtkRenderer* renderer)
 {
   auto model = this->Loader->GetInternalModel();
+  if (!model)
+  {
+    vtkErrorMacro("The GLTF model is nullptr, aborting.");
+    this->SetUpdateStatus(vtkImporter::UpdateStatusEnum::FAILURE);
+    return;
+  }
 
-  int scene = model->DefaultScene;
+  const int& scene = model->DefaultScene;
 
   // List of nodes to import
   std::stack<int> nodeIdStack;
+  std::stack<int> dasmParents;
+  int flatActorId = 0;
 
   // Add root nodes to the stack
   for (int nodeId : model->Scenes[scene].Nodes)
   {
     nodeIdStack.push(nodeId);
+    dasmParents.push(0);
   }
 
   this->OutputsDescription = "";
 
   this->Actors.clear();
+  this->ActorCollection->RemoveAllItems();
 
   // Iterate over tree
   while (!nodeIdStack.empty())
   {
     // Get current node
-    int nodeId = nodeIdStack.top();
+    const int nodeId = nodeIdStack.top();
     nodeIdStack.pop();
-    vtkGLTFDocumentLoader::Node& node = model->Nodes[nodeId];
+    const auto& node = model->Nodes[nodeId];
+
+    // Add this node into the scene hierarchy
+    const int dasmParent = dasmParents.top();
+    dasmParents.pop();
+    std::string dasmNodeName;
+    if (!node.Name.empty())
+    {
+      dasmNodeName = vtkDataAssembly::MakeValidNodeName(node.Name.c_str());
+    }
+    else
+    {
+      dasmNodeName = "node" + std::to_string(nodeId);
+    }
+    const int dasmNode = this->SceneHierarchy->AddNode(dasmNodeName.c_str(), dasmParent);
 
     // Import node's geometry
     if (node.Mesh >= 0)
     {
       auto mesh = model->Meshes[node.Mesh];
+      int primitiveId = 0;
       for (auto primitive : mesh.Primitives)
       {
         auto pointData = primitive.Geometry->GetPointData();
@@ -483,10 +547,20 @@ void vtkGLTFImporter::ImportActors(vtkRenderer* renderer)
 
         actor->SetMapper(mapper);
         actor->SetUserMatrix(node.GlobalTransform);
-
+        std::string meshNodeName;
         if (!mesh.Name.empty())
         {
-          this->OutputsDescription += mesh.Name + " ";
+          std::string primitiveName = mesh.Name;
+          if (mesh.Primitives.size() > 1)
+          {
+            primitiveName += "_primitive_" + std::to_string(primitiveId++);
+          }
+          this->OutputsDescription += primitiveName;
+          meshNodeName = vtkDataAssembly::MakeValidNodeName(primitiveName.c_str());
+        }
+        else
+        {
+          meshNodeName = "primitive_" + std::to_string(primitiveId++);
         }
         this->OutputsDescription += "Primitive Geometry:\n";
         this->OutputsDescription +=
@@ -495,11 +569,21 @@ void vtkGLTFImporter::ImportActors(vtkRenderer* renderer)
         if (primitive.Material >= 0 &&
           primitive.Material < static_cast<int>(model->Materials.size()))
         {
-          ApplyGLTFMaterialToVTKActor(model, primitive, actor, this->Textures);
+          if (!ApplyGLTFMaterialToVTKActor(model, primitive, actor, this->Textures, this))
+          {
+            vtkErrorMacro("Could not apply GLTF material to VTK actor, aborting.");
+            this->SetUpdateStatus(vtkImporter::UpdateStatusEnum::FAILURE);
+            return;
+          }
         }
         renderer->AddActor(actor);
 
-        this->Actors[nodeId].push_back(actor);
+        this->Actors[nodeId].emplace_back(actor);
+        this->ActorCollection->AddItem(actor);
+        const int actorNode =
+          this->SceneHierarchy->AddNode(meshNodeName.c_str(), /*parent=*/dasmNode);
+        this->SceneHierarchy->SetAttribute(actorNode, "parent_node_name", dasmNodeName.c_str());
+        this->SceneHierarchy->SetAttribute(actorNode, "flat_actor_id", flatActorId++);
 
         this->InvokeEvent(vtkCommand::UpdateDataEvent);
       }
@@ -509,6 +593,7 @@ void vtkGLTFImporter::ImportActors(vtkRenderer* renderer)
     for (int childNodeId : node.Children)
     {
       nodeIdStack.push(childNodeId);
+      dasmParents.push(dasmNode);
     }
   }
 
@@ -519,6 +604,12 @@ void vtkGLTFImporter::ImportActors(vtkRenderer* renderer)
 void vtkGLTFImporter::ImportCameras(vtkRenderer* renderer)
 {
   auto model = this->Loader->GetInternalModel();
+  if (!model)
+  {
+    vtkErrorMacro("The GLTF model is nullptr, aborting.");
+    this->SetUpdateStatus(vtkImporter::UpdateStatusEnum::FAILURE);
+    return;
+  }
 
   int scene = model->DefaultScene;
 
@@ -531,11 +622,14 @@ void vtkGLTFImporter::ImportCameras(vtkRenderer* renderer)
     nodeIdStack.push(nodeId);
   }
 
+  this->CameraCollection->RemoveAllItems();
   this->Cameras.clear();
   for (size_t i = 0; i < model->Cameras.size(); i++)
   {
     vtkGLTFDocumentLoader::Camera const& camera = model->Cameras[i];
-    this->Cameras[static_cast<int>(i)] = GLTFCameraToVTKCamera(camera);
+    vtkSmartPointer<vtkCamera> vtkCam = GLTFCameraToVTKCamera(camera);
+    this->Cameras[static_cast<int>(i)] = vtkCam;
+    this->CameraCollection->AddItem(vtkCam);
   }
 
   // Iterate over tree
@@ -562,7 +656,9 @@ void vtkGLTFImporter::ImportCameras(vtkRenderer* renderer)
   // update enabled camera
   if (this->EnabledCamera >= this->GetNumberOfCameras())
   {
-    vtkErrorMacro("Camera index " << this->EnabledCamera << "is invalid");
+    vtkWarningMacro("Camera index "
+      << this->EnabledCamera
+      << "is invalid, cameras have been imported but none have been enabled.");
     this->EnabledCamera = -1;
     return;
   }
@@ -577,18 +673,25 @@ void vtkGLTFImporter::ImportCameras(vtkRenderer* renderer)
 vtkIdType vtkGLTFImporter::GetNumberOfCameras()
 {
   auto model = this->Loader->GetInternalModel();
+  if (!model)
+  {
+    vtkErrorMacro("The GLTF model is nullptr, aborting.");
+    return 0;
+  }
+
   return model->Cameras.size();
 }
 
 //------------------------------------------------------------------------------
 std::string vtkGLTFImporter::GetCameraName(vtkIdType camIndex)
 {
-  auto model = this->Loader->GetInternalModel();
   if (camIndex < 0 || camIndex >= this->GetNumberOfCameras())
   {
     vtkErrorMacro("Camera index invalid");
     return "";
   }
+  auto model = this->Loader->GetInternalModel();
+  assert(model);
   return model->Cameras[camIndex].Name;
 }
 
@@ -614,6 +717,15 @@ void vtkGLTFImporter::ImportLights(vtkRenderer* renderer)
   std::stack<int> nodeIdStack;
 
   const auto& model = this->Loader->GetInternalModel();
+  if (!model)
+  {
+    vtkErrorMacro("The GLTF model is nullptr, aborting.");
+    this->SetUpdateStatus(vtkImporter::UpdateStatusEnum::FAILURE);
+    return;
+  }
+
+  this->LightCollection->RemoveAllItems();
+
   const auto& lights = model->ExtensionMetaData.KHRLightsPunctualMetaData.Lights;
 
   // Add root nodes to the stack
@@ -662,6 +774,7 @@ void vtkGLTFImporter::ImportLights(vtkRenderer* renderer)
           break;
       }
       renderer->AddLight(light);
+      this->LightCollection->AddItem(light);
     }
 
     // Add node's children to stack
@@ -673,13 +786,17 @@ void vtkGLTFImporter::ImportLights(vtkRenderer* renderer)
 }
 
 //----------------------------------------------------------------------------
-void vtkGLTFImporter::UpdateTimeStep(double timestep)
+bool vtkGLTFImporter::UpdateAtTimeValue(double timeValue)
 {
   for (int animationId = 0; animationId < this->GetNumberOfAnimations(); animationId++)
   {
     if (this->EnabledAnimations[animationId])
     {
-      this->Loader->ApplyAnimation(static_cast<float>(timestep), animationId);
+      if (!this->Loader->ApplyAnimation(static_cast<float>(timeValue), animationId))
+      {
+        vtkErrorMacro("Error applying animation");
+        return false;
+      }
     }
   }
   this->Loader->BuildGlobalTransforms();
@@ -687,12 +804,21 @@ void vtkGLTFImporter::UpdateTimeStep(double timestep)
   this->ImportCameras(this->Renderer);
 
   this->ApplySkinningMorphing();
+
+  return this->GetUpdateStatus() == vtkImporter::UpdateStatusEnum::SUCCESS;
 }
 
 //----------------------------------------------------------------------------
 void vtkGLTFImporter::ApplySkinningMorphing()
 {
   const auto& model = this->Loader->GetInternalModel();
+  if (!model)
+  {
+    vtkErrorMacro("The GLTF model is nullptr, aborting.");
+    this->SetUpdateStatus(vtkImporter::UpdateStatusEnum::FAILURE);
+    return;
+  }
+
   int scene = model->DefaultScene;
 
   // List of nodes to import
@@ -772,7 +898,14 @@ void vtkGLTFImporter::ApplySkinningMorphing()
 //----------------------------------------------------------------------------
 vtkIdType vtkGLTFImporter::GetNumberOfAnimations()
 {
-  return static_cast<vtkIdType>(this->Loader->GetInternalModel()->Animations.size());
+  const auto& model = this->Loader->GetInternalModel();
+  if (!model)
+  {
+    vtkErrorMacro("The GLTF model is nullptr, aborting.");
+    return 0;
+  }
+
+  return static_cast<vtkIdType>(model->Animations.size());
 }
 
 //----------------------------------------------------------------------------
@@ -780,7 +913,9 @@ std::string vtkGLTFImporter::GetAnimationName(vtkIdType animationIndex)
 {
   if (animationIndex >= 0 && animationIndex < this->GetNumberOfAnimations())
   {
-    return this->Loader->GetInternalModel()->Animations[animationIndex].Name;
+    const auto& model = this->Loader->GetInternalModel();
+    assert(model);
+    return model->Animations[animationIndex].Name;
   }
   return "";
 }
@@ -813,21 +948,27 @@ bool vtkGLTFImporter::IsAnimationEnabled(vtkIdType animationIndex)
 bool vtkGLTFImporter::GetTemporalInformation(vtkIdType animationIndex, double frameRate,
   int& nbTimeSteps, double timeRange[2], vtkDoubleArray* timeSteps)
 {
-  nbTimeSteps = 0;
   if (animationIndex < this->GetNumberOfAnimations())
   {
+    const auto& model = this->Loader->GetInternalModel();
+    assert(model);
+
     timeRange[0] = 0;
-    timeRange[1] = this->Loader->GetInternalModel()->Animations[animationIndex].Duration;
+    timeRange[1] = model->Animations[animationIndex].Duration;
 
-    timeSteps->SetNumberOfComponents(1);
-    timeSteps->SetNumberOfTuples(0);
-
-    std::vector<double> ts;
-    double period = (1.0 / frameRate);
-    for (double i = timeRange[0]; i < timeRange[1]; i += period)
+    if (frameRate > 0)
     {
-      timeSteps->InsertNextTuple(&i);
-      nbTimeSteps++;
+      nbTimeSteps = 0;
+      timeSteps->SetNumberOfComponents(1);
+      timeSteps->SetNumberOfTuples(0);
+
+      std::vector<double> ts;
+      double period = (1.0 / frameRate);
+      for (double i = timeRange[0]; i < timeRange[1]; i += period)
+      {
+        timeSteps->InsertNextTuple(&i);
+        nbTimeSteps++;
+      }
     }
     return true;
   }
@@ -852,3 +993,5 @@ vtkSmartPointer<vtkCamera> vtkGLTFImporter::GetCamera(unsigned int id)
   }
   return it->second;
 }
+
+VTK_ABI_NAMESPACE_END
