@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkScalarBarActor.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkScalarBarActor.h"
 #include "vtkScalarBarActorInternal.h"
 
@@ -27,6 +15,7 @@
 #include "vtkMathTextUtilities.h"
 #include "vtkNew.h"
 #include "vtkObjectFactory.h"
+#include "vtkPiecewiseFunction.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 #include "vtkPolyDataMapper2D.h"
@@ -43,16 +32,17 @@
 #include "vtkWindow.h"
 
 #include <map>
-#include <set>
 #include <vector>
 
 #include <cstdio> // for snprintf
 
 #undef VTK_DBG_LAYOUT
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkScalarBarActor);
 
 vtkCxxSetObjectMacro(vtkScalarBarActor, LookupTable, vtkScalarsToColors);
+vtkCxxSetObjectMacro(vtkScalarBarActor, OpacityFunction, vtkPiecewiseFunction);
 vtkCxxSetObjectMacro(vtkScalarBarActor, AnnotationTextProperty, vtkTextProperty);
 vtkCxxSetObjectMacro(vtkScalarBarActor, LabelTextProperty, vtkTextProperty);
 vtkCxxSetObjectMacro(vtkScalarBarActor, TitleTextProperty, vtkTextProperty);
@@ -68,6 +58,7 @@ vtkScalarBarActor::vtkScalarBarActor()
 {
   this->P = new vtkScalarBarActorInternal;
   this->LookupTable = nullptr;
+  this->OpacityFunction = nullptr;
   this->Position2Coordinate->SetValue(0.17, 0.8);
 
   this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
@@ -266,6 +257,8 @@ vtkScalarBarActor::vtkScalarBarActor()
   this->DrawColorBar = 1;
   this->DrawTickLabels = 1;
   this->UnconstrainedFontSize = false;
+
+  this->ForceVerticalTitle = false;
 }
 
 //------------------------------------------------------------------------------
@@ -353,6 +346,7 @@ vtkScalarBarActor::~vtkScalarBarActor()
   this->ComponentTitle = nullptr;
 
   this->SetLookupTable(nullptr);
+  this->SetOpacityFunction(nullptr);
   this->SetAnnotationTextProperty(nullptr);
   if (this->CustomLabels != nullptr)
   {
@@ -529,7 +523,8 @@ int vtkScalarBarActor::RebuildLayoutIfNeeded(vtkViewport* viewport)
     this->LabelTextProperty->GetMTime() > this->BuildTime ||
     this->TitleTextProperty->GetMTime() > this->BuildTime ||
     this->BackgroundProperty->GetMTime() > this->BuildTime ||
-    this->FrameProperty->GetMTime() > this->BuildTime)
+    this->FrameProperty->GetMTime() > this->BuildTime ||
+    (this->OpacityFunction && this->OpacityFunction->GetMTime() > this->BuildTime))
 
   {
     this->RebuildLayout(viewport);
@@ -1120,22 +1115,40 @@ void vtkScalarBarActor::LayoutTitle()
   }
 
   int targetWidth, targetHeight;
-  // Title spans entire width of frame at top, regardless of orientation.
-  targetWidth = static_cast<int>(this->P->Frame.Size[this->P->TL[0]]) - 2 * this->TextPad;
-  // Height is either: at most half the frame height or
-  // a fixed portion of the frame remaining after subtracting the
-  // scalar bar's thickness.
-  //
-  // When laid out horizontally, ticks share vertical space with title.
-  // We want the title to be larger (18pt vs 14pt).
-  targetHeight = static_cast<int>(
-    (this->Orientation == VTK_ORIENT_VERTICAL || this->LookupTable->GetIndexedLookup())
-      ? ceil(this->P->Frame.Size[this->P->TL[1]] / 2. - this->TextPad)
-      : (this->P->Frame.Size[0] - this->P->ScalarBarBox.Size[0] -
-          (this->TextPosition == SucceedScalarBar ? this->P->ScalarBarBox.Posn[this->P->TL[0]]
-                                                  : 0) -
-          this->TextPad) *
-        this->TitleRatio);
+  if (!this->ForceVerticalTitle)
+  {
+    // Title spans entire width of frame at top, regardless of orientation.
+    targetWidth = static_cast<int>(this->P->Frame.Size[this->P->TL[0]]) - 2 * this->TextPad;
+  }
+  else
+  {
+    targetWidth = static_cast<int>(
+      (this->P->Frame.Size[0] - this->P->ScalarBarBox.Size[0] - this->TextPad) * this->TitleRatio);
+  }
+
+  if (!this->ForceVerticalTitle)
+  {
+    // Height is either: at most half the frame height or
+    // a fixed portion of the frame remaining after subtracting the
+    // scalar bar's thickness.
+    //
+    // When laid out horizontally, ticks share vertical space with title.
+    // We want the title to be larger (18pt vs 14pt).
+    targetHeight = static_cast<int>(
+      (this->Orientation == VTK_ORIENT_VERTICAL || this->LookupTable->GetIndexedLookup())
+        ? ceil(this->P->Frame.Size[this->P->TL[1]] / 2. - this->TextPad)
+        : (this->P->Frame.Size[0] - this->P->ScalarBarBox.Size[0] -
+            (this->TextPosition == SucceedScalarBar ? this->P->ScalarBarBox.Posn[this->P->TL[0]]
+                                                    : 0) -
+            this->TextPad) *
+          this->TitleRatio);
+  }
+  else
+  {
+    targetHeight = static_cast<int>(this->P->Frame.Size[1] / 3.0);
+  }
+
+  this->TitleActor->SetOrientation(this->ForceVerticalTitle ? 90 : 0);
 
   if (this->UnconstrainedFontSize)
   {
@@ -1156,29 +1169,48 @@ void vtkScalarBarActor::LayoutTitle()
     this->P->TitleBox.Size[this->P->TL[i]] = static_cast<int>(ceil(titleSize[i]));
   }
 
-  this->P->TitleBox.Posn[0] =
-    this->P->Frame.Posn[0] + (this->P->Frame.Size[this->P->TL[0]] - titleSize[0]) / 2;
-  this->P->TitleBox.Posn[1] =
-    static_cast<int>(this->P->Frame.Posn[1] + this->P->Frame.Size[this->P->TL[1]]);
-  if (this->Orientation == VTK_ORIENT_VERTICAL ||
-    this->TextPosition == vtkScalarBarActor::SucceedScalarBar)
+  if (this->ForceVerticalTitle)
   {
-    this->P->TitleBox.Posn[1] -= this->P->TitleBox.Size[this->P->TL[1]] + this->TextPad +
-      static_cast<int>(this->FrameProperty->GetLineWidth());
+    this->P->TitleBox.Posn[0] = this->TextPosition == vtkScalarBarActor::PrecedeScalarBar
+      ? this->P->Frame.Posn[0]
+      : this->P->Frame.Posn[0] + this->P->Frame.Size[this->P->TL[0]] -
+        (this->P->TitleBox.Size[this->P->TL[0]] + this->TextPad);
+    this->P->TitleBox.Posn[1] = this->P->Frame.Posn[1] +
+      (this->P->Frame.Size[this->P->TL[1]] - this->P->TitleBox.Size[this->P->TL[1]]) / 2;
   }
   else
   {
-    this->P->TitleBox.Posn[1] = this->P->Frame.Posn[1] + this->TextPad -
-      static_cast<int>(this->FrameProperty->GetLineWidth());
+    this->P->TitleBox.Posn[0] =
+      this->P->Frame.Posn[0] + (this->P->Frame.Size[this->P->TL[0]] - titleSize[0]) / 2;
+    this->P->TitleBox.Posn[1] =
+      static_cast<int>(this->P->Frame.Posn[1] + this->P->Frame.Size[this->P->TL[1]]);
+    if (this->Orientation == VTK_ORIENT_VERTICAL ||
+      this->TextPosition == vtkScalarBarActor::SucceedScalarBar)
+    {
+      this->P->TitleBox.Posn[1] -= this->P->TitleBox.Size[this->P->TL[1]] + this->TextPad +
+        static_cast<int>(this->FrameProperty->GetLineWidth());
+    }
+    else
+    {
+      this->P->TitleBox.Posn[1] = this->P->Frame.Posn[1] + this->TextPad -
+        static_cast<int>(this->FrameProperty->GetLineWidth());
+    }
   }
 }
 
 //------------------------------------------------------------------------------
 void vtkScalarBarActor::ComputeScalarBarLength()
 {
-  this->P->ScalarBarBox.Size[1] = this->Orientation == VTK_ORIENT_VERTICAL
-    ? this->P->Frame.Size[1] - this->P->TitleBox.Size[1] - this->VerticalTitleSeparation
-    : this->P->Frame.Size[1];
+  if (!this->ForceVerticalTitle)
+  {
+    this->P->ScalarBarBox.Size[1] = this->Orientation == VTK_ORIENT_VERTICAL
+      ? this->P->Frame.Size[1] - this->P->TitleBox.Size[1] - this->VerticalTitleSeparation
+      : this->P->Frame.Size[1];
+  }
+  else
+  {
+    this->P->ScalarBarBox.Size[1] = this->P->Frame.Size[1];
+  }
 
   // The scalar bar does not include the Nan Swatch, the Below Range Swatch and
   // the Above Range Swatch.
@@ -1234,7 +1266,7 @@ void vtkScalarBarActor::LayoutTicks()
   this->P->TextActorAnchors.resize(this->NumberOfLabelsBuilt);
 
   // Does this map have its scale set to log?
-  int isLogTable = this->LookupTable->UsingLogScale();
+  vtkTypeBool isLogTable = this->LookupTable->UsingLogScale();
   if (isLogTable)
   {
     offset = log10(range[0]);
@@ -1332,6 +1364,11 @@ void vtkScalarBarActor::LayoutTicks()
       // lowered by box constraints, but we won't bother:
       this->P->TickBox.Size[1] = this->P->Frame.Size[1] - this->P->TitleBox.Size[1] -
         3 * this->TextPad - this->VerticalTitleSeparation;
+      if (this->ForceVerticalTitle)
+      {
+        this->P->TickBox.Size[0] -= this->P->TitleBox.Size[0];
+        this->P->TickBox.Size[1] += this->P->TitleBox.Size[1];
+      }
       // Tick box height also reduced by NaN swatch size, if present:
       if (this->DrawNanAnnotation)
       {
@@ -1352,6 +1389,10 @@ void vtkScalarBarActor::LayoutTicks()
       if (this->TextPosition == vtkScalarBarActor::PrecedeScalarBar)
       {
         this->P->TickBox.Posn[0] = this->TextPad;
+        if (this->ForceVerticalTitle)
+        {
+          this->P->TickBox.Posn[0] += this->P->TitleBox.Size[0];
+        }
       }
       else
       {
@@ -1592,15 +1633,24 @@ void vtkScalarBarActor::ConfigureScalarBar()
     double rgbval;
     if (this->LookupTable->UsingLogScale())
     {
-      rgbval = log10(range[0]) + i * (log10(range[1]) - log10(range[0])) / this->P->NumColors;
+      rgbval =
+        log10(range[0]) + (i + 0.5) * (log10(range[1]) - log10(range[0])) / this->P->NumColors;
       rgbval = pow(10.0, rgbval);
     }
     else
     {
-      rgbval = range[0] + (range[1] - range[0]) * (i / static_cast<double>(this->P->NumColors));
+      rgbval =
+        range[0] + (range[1] - range[0]) * ((i + 0.5) / static_cast<double>(this->P->NumColors));
     }
     lut->GetColor(rgbval, rgba);
-    rgba[3] = lut->GetOpacity(rgbval);
+    if (this->OpacityFunction)
+    {
+      rgba[3] = this->OpacityFunction->GetValue(rgbval);
+    }
+    else
+    {
+      rgba[3] = lut->GetOpacity(rgbval);
+    }
     // write into array directly
     rgb = this->P->SwatchColors->GetPointer(nComponents * i);
     rgb[0] = static_cast<unsigned char>(rgba[0] * 255.);
@@ -1642,11 +1692,19 @@ void vtkScalarBarActor::ConfigureScalarBar()
 //------------------------------------------------------------------------------
 void vtkScalarBarActor::ConfigureTitle()
 {
-  this->TitleActor->SetPosition(
-    this->P->TitleBox.Posn[0] + this->P->TitleBox.Size[this->P->TL[0]] / 2,
-    this->TitleActor->GetTextProperty()->GetVerticalJustification() == VTK_TEXT_BOTTOM
-      ? this->P->TitleBox.Posn[1]
-      : this->P->TitleBox.Posn[1] + this->P->TitleBox.Size[this->P->TL[1]]);
+  if (!this->ForceVerticalTitle)
+  {
+    this->TitleActor->SetPosition(
+      this->P->TitleBox.Posn[0] + this->P->TitleBox.Size[this->P->TL[0]] / 2,
+      this->TitleActor->GetTextProperty()->GetVerticalJustification() == VTK_TEXT_BOTTOM
+        ? this->P->TitleBox.Posn[1]
+        : this->P->TitleBox.Posn[1] + this->P->TitleBox.Size[this->P->TL[1]]);
+  }
+  else
+  {
+    this->TitleActor->SetPosition(this->P->TitleBox.Posn[0],
+      this->P->TitleBox.Posn[1] + this->P->TitleBox.Size[this->P->TL[1]] / 2);
+  }
 }
 
 //------------------------------------------------------------------------------
@@ -1848,8 +1906,7 @@ void vtkScalarBarActor::ConfigureAnnotations()
   this->P->AnnotationAnchors.resize(numNotes);
   this->P->AnnotationColors.resize(numNotes);
   int i = 0;
-  std::map<double, vtkStdString>::iterator it;
-  for (it = this->P->Labels.begin(); it != this->P->Labels.end(); ++it, ++i)
+  for (auto it = this->P->Labels.begin(); it != this->P->Labels.end(); ++it, ++i)
   {
     this->P->AnnotationAnchors[i] = it->first;
     this->P->AnnotationColors[i] = this->P->LabelColors[it->first];
@@ -2090,7 +2147,7 @@ int vtkScalarBarActor::MapAnnotationLabels(
   {
     for (int i = 0; i < numNotes; ++i)
     {
-      vtkStdString label = lkup->GetAnnotation(i);
+      std::string label = lkup->GetAnnotation(i);
       lkup->GetAnnotationColor(lkup->GetAnnotatedValue(i), fltCol.GetData());
       double x;
       bool canPositionLabel = !label.empty();
@@ -2194,25 +2251,28 @@ int vtkScalarBarActor::PlaceAnnotationsVertically(double barX, double barY,
   }
 
 #define VTK_ANN_VLAYOUT(j, dir, delt)                                                              \
-  ctr = this->P->AnnotationAnchors[j];                                                             \
-  ll[0] = lpts->InsertNextPoint(xl0, ctr, 0.);                                                     \
-  this->P->AnnotationLabels[j]->GetSize(this->P->Viewport, tsz);                                   \
-  hh = (tsz[1] + pad) / 2.; /* label half-height, incl. padding */                                 \
-  if (((dir) < 0 && ctr + hh > dnCum) || ((dir) > 0 && ctr - hh < upCum))                          \
-    ctr = (delt) + (dir)*hh;                                                                       \
-  this->P->AnnotationLabels[j]->GetTextProperty()->SetJustification(                               \
-    this->TextPosition == PrecedeScalarBar ? VTK_TEXT_LEFT : VTK_TEXT_RIGHT);                      \
-  this->P->AnnotationLabels[j]->GetTextProperty()->SetVerticalJustificationToCentered();           \
-  this->P->AnnotationLabels[j]->SetPosition(barX +                                                 \
-      (this->TextPosition == PrecedeScalarBar ? 1 : -1) * (pad + this->AnnotationLeaderPadding),   \
-    ctr);                                                                                          \
-  ll[1] = lpts->InsertNextPoint(xl1, ctr, 0.);                                                     \
-  llines->InsertNextCell(2, ll);                                                                   \
-  llcolors->InsertNextTypedTuple(this->P->AnnotationColors[j].GetData());                          \
-  if (upCum < ctr + hh)                                                                            \
-    upCum = ctr + hh;                                                                              \
-  if (dnCum > ctr - hh)                                                                            \
-    dnCum = ctr - hh;
+  do                                                                                               \
+  {                                                                                                \
+    ctr = this->P->AnnotationAnchors[j];                                                           \
+    ll[0] = lpts->InsertNextPoint(xl0, ctr, 0.);                                                   \
+    this->P->AnnotationLabels[j]->GetSize(this->P->Viewport, tsz);                                 \
+    hh = (tsz[1] + pad) / 2.; /* label half-height, incl. padding */                               \
+    if (((dir) < 0 && ctr + hh > dnCum) || ((dir) > 0 && ctr - hh < upCum))                        \
+      ctr = (delt) + (dir)*hh;                                                                     \
+    this->P->AnnotationLabels[j]->GetTextProperty()->SetJustification(                             \
+      this->TextPosition == PrecedeScalarBar ? VTK_TEXT_LEFT : VTK_TEXT_RIGHT);                    \
+    this->P->AnnotationLabels[j]->GetTextProperty()->SetVerticalJustificationToCentered();         \
+    this->P->AnnotationLabels[j]->SetPosition(barX +                                               \
+        (this->TextPosition == PrecedeScalarBar ? 1 : -1) * (pad + this->AnnotationLeaderPadding), \
+      ctr);                                                                                        \
+    ll[1] = lpts->InsertNextPoint(xl1, ctr, 0.);                                                   \
+    llines->InsertNextCell(2, ll);                                                                 \
+    llcolors->InsertNextTypedTuple(this->P->AnnotationColors[j].GetData());                        \
+    if (upCum < ctr + hh)                                                                          \
+      upCum = ctr + hh;                                                                            \
+    if (dnCum > ctr - hh)                                                                          \
+      dnCum = ctr - hh;                                                                            \
+  } while (false)
 
   int numNotes = static_cast<int>(this->P->AnnotationLabels.size());
   vtkPoints* lpts = vtkPoints::New();
@@ -2551,12 +2611,15 @@ int vtkScalarBarActor::PlaceAnnotationsHorizontally(
   }
 
 #define VTK_ANN_HLAYOUT(j, placer)                                                                 \
-  this->P->AnnotationLabels[j]->GetTextProperty()->SetJustification(                               \
-    (placer).Places[j].Justification);                                                             \
-  this->P->AnnotationLabels[j]->GetTextProperty()->SetVerticalJustification(                       \
-    (placer).Dir > 0 ? VTK_TEXT_BOTTOM : VTK_TEXT_TOP);                                            \
-  this->P->AnnotationLabels[j]->SetPosition((placer).Places[j].Anchor);                            \
-  (placer).AddBrokenLeader(j, lpts, llines, llcolors, this->P->AnnotationColors[j]);
+  do                                                                                               \
+  {                                                                                                \
+    this->P->AnnotationLabels[j]->GetTextProperty()->SetJustification(                             \
+      (placer).Places[j].Justification);                                                           \
+    this->P->AnnotationLabels[j]->GetTextProperty()->SetVerticalJustification(                     \
+      (placer).Dir > 0 ? VTK_TEXT_BOTTOM : VTK_TEXT_TOP);                                          \
+    this->P->AnnotationLabels[j]->SetPosition((placer).Places[j].Anchor);                          \
+    (placer).AddBrokenLeader(j, lpts, llines, llcolors, this->P->AnnotationColors[j]);             \
+  } while (false)
 
   int numNotes = static_cast<int>(this->P->AnnotationLabels.size());
   bool precede = this->TextPosition == vtkScalarBarActor::PrecedeScalarBar;
@@ -2617,3 +2680,4 @@ int vtkScalarBarActor::PlaceAnnotationsHorizontally(
   llcolors->Delete();
   return numNotes;
 }
+VTK_ABI_NAMESPACE_END
