@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkWin32RenderWindowInteractor.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include <cmath>
 #include <cstdio>
 #include <cstdlib>
@@ -25,11 +13,12 @@
 #define _WIN32_WINNT 0x0601 // for touch support, 0x0601 means target Windows 7 or later
 #endif
 
+#include "vtkHardwareWindow.h"
 #include "vtkRenderWindow.h"
+#include "vtkStringArray.h"
 #include "vtkWin32RenderWindowInteractor.h"
 #include "vtkWindows.h"
-
-#include "vtkStringArray.h"
+#include "vtksys/Encoding.hxx"
 
 #include <shellapi.h> // for drag and drop
 #include <winuser.h>  // for touch support
@@ -48,10 +37,12 @@
 // sees it in the friend decl in vtkWin32RenderWindowInteractor, but
 // GCC needs to see the declaration beforehand. It has to do with the
 // CALLBACK attribute.
+VTK_ABI_NAMESPACE_BEGIN
 VTKRENDERINGUI_EXPORT LRESULT CALLBACK vtkHandleMessage(HWND, UINT, WPARAM, LPARAM);
 VTKRENDERINGUI_EXPORT LRESULT CALLBACK vtkHandleMessage2(
   HWND, UINT, WPARAM, LPARAM, class vtkWin32RenderWindowInteractor*);
 
+VTK_ABI_NAMESPACE_END
 #include "vtkActor.h"
 #include "vtkCommand.h"
 #include "vtkObjectFactory.h"
@@ -70,7 +61,7 @@ VTKRENDERINGUI_EXPORT LRESULT CALLBACK vtkHandleMessage2(
 
 typedef TOUCHINPUT* PTOUCHINPUT;
 
-//#define HTOUCHINPUT ULONG
+// #define HTOUCHINPUT ULONG
 #define TOUCHEVENTF_MOVE 0x0001
 #define TOUCHEVENTF_DOWN 0x0002
 #define TOUCHEVENTF_UP 0x0004
@@ -78,13 +69,136 @@ typedef bool(WINAPI* RegisterTouchWindowType)(HWND, ULONG);
 typedef bool(WINAPI* GetTouchInputInfoType)(HTOUCHINPUT, UINT, PTOUCHINPUT, int);
 typedef bool(WINAPI* CloseTouchInputHandleType)(HTOUCHINPUT);
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkWin32RenderWindowInteractor);
 
-void (*vtkWin32RenderWindowInteractor::ClassExitMethod)(void*) = (void (*)(void*))nullptr;
-void* vtkWin32RenderWindowInteractor::ClassExitMethodArg = (void*)nullptr;
-void (*vtkWin32RenderWindowInteractor::ClassExitMethodArgDelete)(void*) = (void (*)(void*))nullptr;
+void (*vtkWin32RenderWindowInteractor::ClassExitMethod)(void*) = (void (*)(void*)) nullptr;
+void* vtkWin32RenderWindowInteractor::ClassExitMethodArg = nullptr;
+void (*vtkWin32RenderWindowInteractor::ClassExitMethodArgDelete)(void*) = (void (*)(void*)) nullptr;
 
-//----------------------------------------------------------------------------
+namespace
+{
+//-------------------------------------------------------------
+// Virtual Key Code to Unix KeySym Conversion
+//-------------------------------------------------------------
+
+// clang-format off
+// this unicode code to keysym table is meant to provide keysym similar to XLookupString,
+// for Basic Latin and Latin1 unicode blocks.
+// Generated from xlib/X11/keysymdef.h
+// Duplicated in Rendering/OpenGL2/vtkCocoaGLView.mm
+static const char* UnicodeToKeySymTable[256] = {
+  // Basic Latin
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+  "space", "exclam", "quotedbl", "numbersign", "dollar", "percent", "ampersand", "apostrophe", "parenleft", "parenright", "asterisk", "plus", "comma", "minus", "period", "slash",
+  "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "colon", "semicolon", "less", "equal", "greater", "question", "at",
+  "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P",
+  "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "bracketleft", "backslash", "bracketright", "asciicircum", "underscore", "grave",
+  "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p",
+  "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "braceleft", "bar", "braceright", "asciitilde", nullptr,
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+
+  // Latin1
+  "nobreakspace", "exclamdown", "cent", "sterling", "currency", "yen", "brokenbar", "section", "diaeresis", "copyright", "ordfeminine", "guillemotleft", "notsign", "hyphen", "registered", "macron",
+  "degree", "plusminus", "twosuperior", "threesuperior", "acute", "mu", "paragraph", "periodcentered", "cedilla", "onesuperior", "masculine", "guillemotright", "onequarter", "onehalf", "threequarters", "questiondown",
+  "Agrave", "Aacute", "Acircumflex", "Atilde", "Adiaeresis", "Aring", "AE", "Ccedilla", "Egrave", "Eacute", "Ecircumflex", "Ediaeresis", "Igrave", "Iacute", "Icircumflex", "Idiaeresis",
+  "ETH", "Ntilde", "Ograve", "Oacute", "Ocircumflex", "Otilde", "Odiaeresis", "multiply", "Ooblique", "Ugrave", "Uacute", "Ucircumflex", "Udiaeresis", "Yacute", "THORN", "ssharp",
+  "agrave", "aacute", "acircumflex", "atilde", "adiaeresis", "aring", "ae", "ccedilla", "egrave", "eacute", "ecircumflex", "ediaeresis", "igrave", "iacute", "icircumflex", "idiaeresis",
+  "eth", "ntilde", "ograve", "oacute", "ocircumflex", "otilde", "odiaeresis", "division", "oslash", "ugrave", "uacute", "ucircumflex", "udiaeresis", "yacute", "thorn", "ydiaeresis"
+};
+
+// This table is meant to provide keysym similar to XLookupString from Windows VKeys (Winuser.h)
+// that are not mapped in the unicode table above.
+static const char* VKeyCodeToKeySymTable[256] = { 
+  nullptr, nullptr, nullptr, "Cancel", nullptr, nullptr, nullptr, nullptr, "BackSpace", "Tab", nullptr, nullptr, "Clear", "Return", nullptr, nullptr, 
+  "Shift_L", "Control_L", "Alt_L", "Pause", "Caps_Lock", nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, "Escape", nullptr, nullptr, nullptr, nullptr, 
+  "space", "Prior", "Next", "End", "Home", "Left", "Up", "Right", "Down", "Select", nullptr, "Execute", "Snapshot", "Insert", "Delete", "Help", 
+  "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+  nullptr, "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", 
+  "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "Win_L", "Win_R", "App", nullptr, nullptr, 
+  "KP_0", "KP_1", "KP_2", "KP_3", "KP_4", "KP_5", "KP_6", "KP_7", "KP_8", "KP_9", "asterisk", "plus", "bar", "minus", "period", "slash", 
+  "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12", "F13", "F14", "F15", "F16", 
+  "F17", "F18", "F19", "F20", "F21", "F22", "F23", "F24", nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
+  "Num_Lock", "Scroll_Lock", nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, 
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr
+};
+// clang-format on
+
+void RecoverModifiersStatus(int& ctrl, int& shift, int& alt)
+{
+  ctrl = (GetKeyState(VK_CONTROL) & 0x8000) != 0;
+  shift = (GetKeyState(VK_SHIFT) & 0x8000) != 0;
+  alt = (GetKeyState(VK_MENU) & 0x8000) != 0;
+}
+
+void RecoverKeyEventInformation(
+  UINT vCode, UINT nFlags, int& ctrl, int& shift, int& alt, char& keyCode, const char*& keySym)
+{
+  RecoverModifiersStatus(ctrl, shift, alt);
+  // WORD is unsigned short
+  WORD nChar = 0;
+  WORD nCharWithoutMod = 0;
+  {
+#ifndef _WIN32_WCE
+    BYTE keyState[256];
+    GetKeyboardState(keyState);
+    if (ToAscii(vCode, nFlags & 0xff, keyState, &nChar, 0) == 0)
+    {
+      nChar = 0;
+    }
+    nCharWithoutMod = nChar;
+    if (ctrl != 0 || alt != 0)
+    {
+      // When using modifiers, recover a keyCode without modifiers
+      // except Shift in order to unsure behavior consistency with
+      // other OSes.
+      keyState[VK_CONTROL] = 0;
+      keyState[VK_MENU] = 0;
+      if (ToAscii(vCode, nFlags & 0xff, keyState, &nCharWithoutMod, 0) == 0)
+      {
+        nCharWithoutMod = 0;
+      }
+    }
+#endif
+  }
+
+  // keyCode is the modified one except when it is 0
+  // in that case, fallback on the version without modifiers
+  keyCode = static_cast<char>(nChar);
+  if (keyCode == 0)
+  {
+    keyCode = static_cast<char>(nCharWithoutMod);
+  }
+
+  // Try to recover any valid keySym
+  if (nChar < 256)
+  {
+    keySym = UnicodeToKeySymTable[nChar];
+  }
+  if (keySym == nullptr && nCharWithoutMod < 256)
+  {
+    keySym = UnicodeToKeySymTable[nCharWithoutMod];
+  }
+  if (keySym == nullptr && vCode < 256)
+  {
+    keySym = VKeyCodeToKeySymTable[vCode];
+  }
+  if (keySym == nullptr)
+  {
+    keySym = "None";
+  }
+}
+
+}
+
+//------------------------------------------------------------------------------
 // Construct object so that light follows camera motion.
 vtkWin32RenderWindowInteractor::vtkWin32RenderWindowInteractor()
 {
@@ -98,7 +212,7 @@ vtkWin32RenderWindowInteractor::vtkWin32RenderWindowInteractor()
 #endif
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkWin32RenderWindowInteractor::~vtkWin32RenderWindowInteractor()
 {
   vtkRenderWindow* tmp;
@@ -129,7 +243,7 @@ vtkWin32RenderWindowInteractor::~vtkWin32RenderWindowInteractor()
 #endif
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkWin32RenderWindowInteractor::ProcessEvents()
 {
   // No need to do anything if this is a 'mapped' interactor
@@ -146,7 +260,7 @@ void vtkWin32RenderWindowInteractor::ProcessEvents()
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkWin32RenderWindowInteractor::StartEventLoop()
 {
   // No need to do anything if this is a 'mapped' interactor
@@ -165,7 +279,7 @@ void vtkWin32RenderWindowInteractor::StartEventLoop()
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // Begin processing keyboard strokes.
 void vtkWin32RenderWindowInteractor::Initialize()
 {
@@ -189,13 +303,25 @@ void vtkWin32RenderWindowInteractor::Initialize()
   ren->End();
   size = ren->GetSize();
   ren->GetPosition();
+
   this->WindowId = (HWND)(ren->GetGenericWindowId());
+  if (this->HardwareWindow)
+  {
+    this->WindowId = (HWND)(this->HardwareWindow->GetGenericWindowId());
+    size = this->HardwareWindow->GetSize();
+    vtkSetWindowLong(this->WindowId, sizeof(vtkLONG), (intptr_t)ren);
+  }
+  else
+  {
+    this->WindowId = (HWND)(ren->GetGenericWindowId());
+  }
+
   this->Enable();
   this->Size[0] = size[0];
   this->Size[1] = size[1];
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkWin32RenderWindowInteractor::Enable()
 {
   vtkRenderWindow* ren;
@@ -255,7 +381,7 @@ void vtkWin32RenderWindowInteractor::Enable()
   this->Modified();
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkWin32RenderWindowInteractor::Disable()
 {
   vtkRenderWindow* tmp;
@@ -294,9 +420,14 @@ void vtkWin32RenderWindowInteractor::Disable()
   this->Modified();
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkWin32RenderWindowInteractor::TerminateApp(void)
 {
+  if (this->Done)
+  {
+    return;
+  }
+
   this->Done = true;
 
   // Only post a quit message if Start was called...
@@ -307,7 +438,7 @@ void vtkWin32RenderWindowInteractor::TerminateApp(void)
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::InternalCreateTimer(
   int timerId, int vtkNotUsed(timerType), unsigned long duration)
 {
@@ -316,48 +447,12 @@ int vtkWin32RenderWindowInteractor::InternalCreateTimer(
   return timerId;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::InternalDestroyTimer(int platformTimerId)
 {
   return KillTimer(this->WindowId, platformTimerId);
 }
 
-//-------------------------------------------------------------
-// Virtual Key Code to Unix KeySym Conversion
-//-------------------------------------------------------------
-
-// this ascii code to keysym table is meant to mimic Tk
-
-static const char* AsciiToKeySymTable[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "space", "exclam", "quotedbl", "numbersign", "dollar",
-  "percent", "ampersand", "quoteright", "parenleft", "parenright", "asterisk", "plus", "comma",
-  "minus", "period", "slash", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "colon",
-  "semicolon", "less", "equal", "greater", "question", "at", "A", "B", "C", "D", "E", "F", "G", "H",
-  "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z",
-  "bracketleft", "backslash", "bracketright", "asciicircum", "underscore", "quoteleft", "a", "b",
-  "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u",
-  "v", "w", "x", "y", "z", "braceleft", "bar", "braceright", "asciitilde", "Delete", 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-
-// this virtual key code to keysym table is meant to mimic Tk
-
-static const char* VKeyCodeToKeySymTable[] = { 0, 0, 0, "Cancel", 0, 0, 0, 0, "BackSpace", "Tab", 0,
-  0, "Clear", "Return", 0, 0, "Shift_L", "Control_L", "Alt_L", "Pause", "Caps_Lock", 0, 0, 0, 0, 0,
-  0, "Escape", 0, 0, 0, 0, "space", "Prior", "Next", "End", "Home", "Left", "Up", "Right", "Down",
-  "Select", 0, "Execute", "Snapshot", "Insert", "Delete", "Help", "0", "1", "2", "3", "4", "5", "6",
-  "7", "8", "9", 0, 0, 0, 0, 0, 0, 0, "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l",
-  "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "Win_L", "Win_R", "App", 0,
-  0, "KP_0", "KP_1", "KP_2", "KP_3", "KP_4", "KP_5", "KP_6", "KP_7", "KP_8", "KP_9", "asterisk",
-  "plus", "bar", "minus", "period", "slash", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9",
-  "F10", "F11", "F12", "F13", "F14", "F15", "F16", "F17", "F18", "F19", "F20", "F21", "F22", "F23",
-  "F24", 0, 0, 0, 0, 0, 0, 0, 0, "Num_Lock", "Scroll_Lock", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-
 //-------------------------------------------------------------
 // Event loop handlers
 //-------------------------------------------------------------
@@ -375,7 +470,7 @@ int vtkWin32RenderWindowInteractor::OnMouseMove(HWND hWnd, UINT nFlags, int X, i
   }
 
   this->SetEventInformationFlipY(X, Y, nFlags & MK_CONTROL, nFlags & MK_SHIFT);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   if (!this->MouseInWindow && (X >= 0 && X < this->Size[0] && Y >= 0 && Y < this->Size[1]))
   {
     this->InvokeEvent(vtkCommand::EnterEvent, nullptr);
@@ -391,7 +486,7 @@ int vtkWin32RenderWindowInteractor::OnMouseMove(HWND hWnd, UINT nFlags, int X, i
   return this->InvokeEvent(vtkCommand::MouseMoveEvent, nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnNCMouseMove(HWND, UINT nFlags, int X, int Y)
 {
   if (!this->Enabled || !this->MouseInWindow)
@@ -401,13 +496,13 @@ int vtkWin32RenderWindowInteractor::OnNCMouseMove(HWND, UINT nFlags, int X, int
 
   const int* pos = this->RenderWindow->GetPosition();
   this->SetEventInformationFlipY(X - pos[0], Y - pos[1], nFlags & MK_CONTROL, nFlags & MK_SHIFT);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   const int ret = this->InvokeEvent(vtkCommand::LeaveEvent, nullptr);
   this->MouseInWindow = 0;
   return ret;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnMouseWheelForward(HWND, UINT nFlags, int X, int Y)
 {
   if (!this->Enabled)
@@ -415,11 +510,11 @@ int vtkWin32RenderWindowInteractor::OnMouseWheelForward(HWND, UINT nFlags, int X
     return 0;
   }
   this->SetEventInformationFlipY(X, Y, nFlags & MK_CONTROL, nFlags & MK_SHIFT);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   return this->InvokeEvent(vtkCommand::MouseWheelForwardEvent, nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnMouseWheelBackward(HWND, UINT nFlags, int X, int Y)
 {
   if (!this->Enabled)
@@ -427,11 +522,11 @@ int vtkWin32RenderWindowInteractor::OnMouseWheelBackward(HWND, UINT nFlags, int
     return 0;
   }
   this->SetEventInformationFlipY(X, Y, nFlags & MK_CONTROL, nFlags & MK_SHIFT);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   return this->InvokeEvent(vtkCommand::MouseWheelBackwardEvent, nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnLButtonDown(HWND wnd, UINT nFlags, int X, int Y, int repeat)
 {
   if (!this->Enabled)
@@ -448,11 +543,11 @@ int vtkWin32RenderWindowInteractor::OnLButtonDown(HWND wnd, UINT nFlags, int X,
   SetFocus(wnd);
   SetCapture(wnd);
   this->SetEventInformationFlipY(X, Y, nFlags & MK_CONTROL, nFlags & MK_SHIFT, 0, repeat);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   return this->InvokeEvent(vtkCommand::LeftButtonPressEvent, nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnLButtonUp(HWND, UINT nFlags, int X, int Y)
 {
   if (!this->Enabled)
@@ -465,13 +560,13 @@ int vtkWin32RenderWindowInteractor::OnLButtonUp(HWND, UINT nFlags, int X, int Y)
     return 0;
   }
   this->SetEventInformationFlipY(X, Y, nFlags & MK_CONTROL, nFlags & MK_SHIFT);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   const int ret = this->InvokeEvent(vtkCommand::LeftButtonReleaseEvent, nullptr);
   ReleaseCapture();
   return ret;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnMButtonDown(HWND wnd, UINT nFlags, int X, int Y, int repeat)
 {
   if (!this->Enabled)
@@ -481,11 +576,11 @@ int vtkWin32RenderWindowInteractor::OnMButtonDown(HWND wnd, UINT nFlags, int X,
   SetFocus(wnd);
   SetCapture(wnd);
   this->SetEventInformationFlipY(X, Y, nFlags & MK_CONTROL, nFlags & MK_SHIFT, 0, repeat);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   return this->InvokeEvent(vtkCommand::MiddleButtonPressEvent, nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnMButtonUp(HWND, UINT nFlags, int X, int Y)
 {
   if (!this->Enabled)
@@ -493,13 +588,13 @@ int vtkWin32RenderWindowInteractor::OnMButtonUp(HWND, UINT nFlags, int X, int Y)
     return 0;
   }
   this->SetEventInformationFlipY(X, Y, nFlags & MK_CONTROL, nFlags & MK_SHIFT);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   const int ret = this->InvokeEvent(vtkCommand::MiddleButtonReleaseEvent, nullptr);
   ReleaseCapture();
   return ret;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnRButtonDown(HWND wnd, UINT nFlags, int X, int Y, int repeat)
 {
   if (!this->Enabled)
@@ -509,11 +604,11 @@ int vtkWin32RenderWindowInteractor::OnRButtonDown(HWND wnd, UINT nFlags, int X,
   SetFocus(wnd);
   SetCapture(wnd);
   this->SetEventInformationFlipY(X, Y, nFlags & MK_CONTROL, nFlags & MK_SHIFT, 0, repeat);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   return this->InvokeEvent(vtkCommand::RightButtonPressEvent, nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnRButtonUp(HWND, UINT nFlags, int X, int Y)
 {
   if (!this->Enabled)
@@ -521,13 +616,13 @@ int vtkWin32RenderWindowInteractor::OnRButtonUp(HWND, UINT nFlags, int X, int Y)
     return 0;
   }
   this->SetEventInformationFlipY(X, Y, nFlags & MK_CONTROL, nFlags & MK_SHIFT);
-  this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+  this->SetAltKey((GetKeyState(VK_MENU) & 0x8000) != 0);
   const int ret = this->InvokeEvent(vtkCommand::RightButtonReleaseEvent, nullptr);
   ReleaseCapture();
   return ret;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnSize(HWND, UINT, int X, int Y)
 {
   this->UpdateSize(X, Y);
@@ -538,7 +633,7 @@ int vtkWin32RenderWindowInteractor::OnSize(HWND, UINT, int X, int Y)
   return 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnTimer(HWND, UINT timerId)
 {
   if (!this->Enabled)
@@ -557,92 +652,53 @@ int vtkWin32RenderWindowInteractor::OnTimer(HWND, UINT timerId)
   return ret;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnKeyDown(HWND, UINT vCode, UINT nRepCnt, UINT nFlags)
 {
   if (!this->Enabled)
   {
     return 0;
   }
-  int ctrl = GetKeyState(VK_CONTROL) & (~1);
-  int shift = GetKeyState(VK_SHIFT) & (~1);
-  int alt = GetKeyState(VK_MENU) & (~1);
-  WORD nChar = 0;
-  {
-#ifndef _WIN32_WCE
-    BYTE keyState[256];
-    GetKeyboardState(keyState);
-    if (ToAscii(vCode, nFlags & 0xff, keyState, &nChar, 0) == 0)
-    {
-      nChar = 0;
-    }
-#endif
-  }
-  const char* keysym = AsciiToKeySymTable[(unsigned char)nChar];
-  if (keysym == 0)
-  {
-    keysym = VKeyCodeToKeySymTable[(unsigned char)vCode];
-  }
-  if (keysym == 0)
-  {
-    keysym = "None";
-  }
-  this->SetKeyEventInformation(ctrl, shift, nChar, nRepCnt, keysym);
+  int ctrl, shift, alt;
+  char keyCode;
+  const char* keySym;
+  ::RecoverKeyEventInformation(vCode, nFlags, ctrl, shift, alt, keyCode, keySym);
+  this->SetKeyEventInformation(ctrl, shift, keyCode, nRepCnt, keySym);
   this->SetAltKey(alt);
   return this->InvokeEvent(vtkCommand::KeyPressEvent, nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnKeyUp(HWND, UINT vCode, UINT nRepCnt, UINT nFlags)
 {
   if (!this->Enabled)
   {
     return 0;
   }
-  int ctrl = GetKeyState(VK_CONTROL) & (~1);
-  int shift = GetKeyState(VK_SHIFT) & (~1);
-  int alt = GetKeyState(VK_MENU) & (~1);
-  WORD nChar = 0;
-  {
-    BYTE keyState[256];
-#ifndef _WIN32_WCE
-    GetKeyboardState(keyState);
-    if (ToAscii(vCode, nFlags & 0xff, keyState, &nChar, 0) == 0)
-    {
-      nChar = 0;
-    }
-#endif
-  }
-  const char* keysym = AsciiToKeySymTable[(unsigned char)nChar];
-  if (keysym == 0)
-  {
-    keysym = VKeyCodeToKeySymTable[(unsigned char)vCode];
-  }
-  if (keysym == 0)
-  {
-    keysym = "None";
-  }
-  this->SetKeyEventInformation(ctrl, shift, nChar, nRepCnt, keysym);
+  int ctrl, shift, alt;
+  char keyCode;
+  const char* keySym;
+  ::RecoverKeyEventInformation(vCode, nFlags, ctrl, shift, alt, keyCode, keySym);
+  this->SetKeyEventInformation(ctrl, shift, keyCode, nRepCnt, keySym);
   this->SetAltKey(alt);
   return this->InvokeEvent(vtkCommand::KeyReleaseEvent, nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnChar(HWND, UINT nChar, UINT nRepCnt, UINT)
 {
   if (!this->Enabled)
   {
     return 0;
   }
-  int ctrl = GetKeyState(VK_CONTROL) & (~1);
-  int shift = GetKeyState(VK_SHIFT) & (~1);
-  int alt = GetKeyState(VK_MENU) & (~1);
+  int ctrl, shift, alt;
+  ::RecoverModifiersStatus(ctrl, shift, alt);
   this->SetKeyEventInformation(ctrl, shift, nChar, nRepCnt);
   this->SetAltKey(alt);
   return this->InvokeEvent(vtkCommand::CharEvent, nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnFocus(HWND, UINT)
 {
   if (!this->Enabled)
@@ -661,7 +717,7 @@ int vtkWin32RenderWindowInteractor::OnFocus(HWND, UINT)
   return 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnKillFocus(HWND, UINT)
 {
   if (!this->Enabled)
@@ -679,7 +735,7 @@ int vtkWin32RenderWindowInteractor::OnKillFocus(HWND, UINT)
   return 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnTouch(HWND hWnd, UINT wParam, UINT lParam)
 {
   if (!this->Enabled)
@@ -692,9 +748,9 @@ int vtkWin32RenderWindowInteractor::OnTouch(HWND hWnd, UINT wParam, UINT lParam)
   PTOUCHINPUT pInputs = new TOUCHINPUT[cInputs];
   if (pInputs)
   {
-    int ctrl = GetKeyState(VK_CONTROL) & (~1);
-    int shift = GetKeyState(VK_SHIFT) & (~1);
-    this->SetAltKey(GetKeyState(VK_MENU) & (~1));
+    int ctrl, shift, alt;
+    ::RecoverModifiersStatus(ctrl, shift, alt);
+    this->SetAltKey(alt);
     GetTouchInputInfoType GTII =
       (GetTouchInputInfoType)GetProcAddress(GetModuleHandle(TEXT("user32")), "GetTouchInputInfo");
     if (GTII((HTOUCHINPUT)lParam, cInputs, pInputs, sizeof(TOUCHINPUT)))
@@ -754,7 +810,7 @@ int vtkWin32RenderWindowInteractor::OnTouch(HWND hWnd, UINT wParam, UINT lParam)
   return ret;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkWin32RenderWindowInteractor::OnDropFiles(HWND, WPARAM wParam)
 {
   if (!this->Enabled)
@@ -762,8 +818,6 @@ int vtkWin32RenderWindowInteractor::OnDropFiles(HWND, WPARAM wParam)
     return 0;
   }
 
-  int ret = 0;
-
   HDROP hdrop = reinterpret_cast<HDROP>(wParam);
 
   POINT pt;
@@ -773,11 +827,16 @@ int vtkWin32RenderWindowInteractor::OnDropFiles(HWND, WPARAM wParam)
     location[0] = static_cast<double>(pt.x);
     location[1] = static_cast<double>(this->Size[1] - pt.y - 1); // flip Y
 
+    int ctrl, shift, alt;
+    ::RecoverModifiersStatus(ctrl, shift, alt);
+    this->SetEventInformationFlipY(location[0], location[1], ctrl, shift);
+    this->SetAltKey(alt);
     this->InvokeEvent(vtkCommand::UpdateDropLocationEvent, location);
   }
 
-  UINT cFiles = DragQueryFile(hdrop, 0xFFFFFFFF, nullptr, 0);
+  UINT cFiles = DragQueryFileW(hdrop, 0xFFFFFFFF, nullptr, 0);
 
+  int ret = 0;
   if (cFiles > 0)
   {
     vtkNew<vtkStringArray> filePaths;
@@ -785,17 +844,11 @@ int vtkWin32RenderWindowInteractor::OnDropFiles(HWND, WPARAM wParam)
 
     for (UINT i = 0; i < cFiles; i++)
     {
-      TCHAR file[MAX_PATH];
-      UINT cch = DragQueryFile(hdrop, i, file, MAX_PATH);
+      wchar_t file[MAX_PATH];
+      UINT cch = DragQueryFileW(hdrop, i, file, MAX_PATH);
       if (cch > 0 && cch < MAX_PATH)
       {
-#ifdef UNICODE
-        char cFile[MAX_PATH];
-        wcstombs(cFile, file, cch);
-        filePaths->InsertNextValue(cFile);
-#else
-        filePaths->InsertNextValue(file);
-#endif
+        filePaths->InsertNextValue(vtksys::Encoding::ToNarrow(file));
         ret = 1;
       }
     }
@@ -805,7 +858,7 @@ int vtkWin32RenderWindowInteractor::OnDropFiles(HWND, WPARAM wParam)
   return ret;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // This is only called when InstallMessageProc is true
 LRESULT CALLBACK vtkHandleMessage(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
@@ -834,7 +887,7 @@ LRESULT CALLBACK vtkHandleMessage(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lP
 #define MAKEPOINTS(l) (*((POINTS FAR*)&(l)))
 #endif
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 LRESULT CALLBACK vtkHandleMessage2(
   HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, vtkWin32RenderWindowInteractor* me)
 {
@@ -988,7 +1041,7 @@ LRESULT CALLBACK vtkHandleMessage2(
   return 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // Specify the default function to be called when an interactor needs to exit.
 // This callback is overridden by an instance ExitMethod that is defined.
 void vtkWin32RenderWindowInteractor::SetClassExitMethod(void (*f)(void*), void* arg)
@@ -1010,7 +1063,7 @@ void vtkWin32RenderWindowInteractor::SetClassExitMethod(void (*f)(void*), void*
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // Set the arg delete method.  This is used to free user memory.
 void vtkWin32RenderWindowInteractor::SetClassExitMethodArgDelete(void (*f)(void*))
 {
@@ -1022,7 +1075,7 @@ void vtkWin32RenderWindowInteractor::SetClassExitMethodArgDelete(void (*f)(void*
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkWin32RenderWindowInteractor::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
@@ -1030,7 +1083,7 @@ void vtkWin32RenderWindowInteractor::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "StartedMessageLoop: " << this->StartedMessageLoop << endl;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkWin32RenderWindowInteractor::ExitCallback()
 {
   if (this->HasObserver(vtkCommand::ExitEvent))
@@ -1044,3 +1097,4 @@ void vtkWin32RenderWindowInteractor::ExitCallback()
 
   this->TerminateApp();
 }
+VTK_ABI_NAMESPACE_END
