@@ -1,3 +1,5 @@
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include <algorithm>
 
 #include "vtkBlockSortHelper.h"
@@ -18,8 +20,9 @@
 #include "vtkUnsignedCharArray.h"
 #include "vtkVolumeProperty.h"
 #include "vtkVolumeTexture.h"
-#include "vtk_glew.h"
+#include "vtk_glad.h"
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkVolumeTexture::vtkVolumeTexture()
   : HandleLargeDataTypes(false)
   , InterpolationType(vtkTextureObject::Linear)
@@ -61,8 +64,8 @@ vtkVolumeTexture::~vtkVolumeTexture()
 vtkStandardNewMacro(vtkVolumeTexture);
 
 //------------------------------------------------------------------------------
-bool vtkVolumeTexture::LoadVolume(vtkRenderer* ren, vtkDataSet* data, vtkDataArray* scalars,
-  int const isCell, int const interpolation)
+bool vtkVolumeTexture::LoadVolume(
+  vtkRenderer* ren, vtkDataSet* data, vtkDataArray* scalars, int isCell, int interpolation)
 {
   this->ClearBlocks();
   this->Scalars = scalars;
@@ -166,7 +169,7 @@ bool vtkVolumeTexture::LoadVolume(vtkRenderer* ren, vtkDataSet* data, vtkDataArr
 }
 
 //------------------------------------------------------------------------------
-void vtkVolumeTexture::SetInterpolation(int const interpolation)
+void vtkVolumeTexture::SetInterpolation(int interpolation)
 {
   this->InterpolationType = interpolation;
 
@@ -206,8 +209,7 @@ vtkVolumeTexture::VolumeBlock* vtkVolumeTexture::GetCurrentBlock()
 }
 
 //------------------------------------------------------------------------------
-void vtkVolumeTexture::CreateBlocks(
-  unsigned int const format, unsigned int const internalFormat, int const type)
+void vtkVolumeTexture::CreateBlocks(unsigned int format, unsigned int internalFormat, int type)
 {
   // Pre compute array size
   this->FullSize[0] = this->FullExtent[1] - this->FullExtent[0] + 1;
@@ -281,7 +283,7 @@ vtkVolumeTexture::Size3 vtkVolumeTexture::ComputeBlockSize(int* extent)
 }
 
 //------------------------------------------------------------------------------
-bool vtkVolumeTexture::LoadTexture(int const interpolation, VolumeBlock* volBlock)
+bool vtkVolumeTexture::LoadTexture(int interpolation, VolumeBlock* volBlock)
 {
   int const noOfComponents = this->Scalars->GetNumberOfComponents();
   int scalarType = this->Scalars->GetDataType();
@@ -648,7 +650,7 @@ void vtkVolumeTexture::SplitVolume(vtkImageData* imageData, Size3 const& part)
 
 //------------------------------------------------------------------------------
 void vtkVolumeTexture::GetScaleAndBias(
-  const int scalarType, float* scalarRange, float& scale, float& bias)
+  int scalarType, float* scalarRange, float& scale, float& bias)
 {
   scale = 1.0f;
   bias = 0.0f;
@@ -693,8 +695,8 @@ void vtkVolumeTexture::GetScaleAndBias(
 }
 
 //------------------------------------------------------------------------------
-void vtkVolumeTexture::SelectTextureFormat(unsigned int& format, unsigned int& internalFormat,
-  int& type, int const scalarType, int const noOfComponents)
+void vtkVolumeTexture::SelectTextureFormat(
+  unsigned int& format, unsigned int& internalFormat, int& type, int scalarType, int noOfComponents)
 {
   bool supportsFloat = true;
   this->HandleLargeDataTypes = false;
@@ -835,7 +837,7 @@ void vtkVolumeTexture::UpdateVolume(vtkVolumeProperty* property)
 }
 
 //------------------------------------------------------------------------------
-void vtkVolumeTexture::UpdateInterpolationType(int const interpolation)
+void vtkVolumeTexture::UpdateInterpolationType(int interpolation)
 {
   if (interpolation == VTK_LINEAR_INTERPOLATION &&
     this->InterpolationType != vtkTextureObject::Linear)
@@ -877,7 +879,7 @@ void vtkVolumeTexture::SortBlocksBackToFront(vtkRenderer* ren, vtkMatrix4x4* vol
 }
 
 //------------------------------------------------------------------------------
-void vtkVolumeTexture::SetPartitions(int const x, int const y, int const z)
+void vtkVolumeTexture::SetPartitions(int x, int y, int z)
 {
   if (x > 0 && y > 0 && z > 0)
   {
@@ -921,7 +923,7 @@ void vtkVolumeTexture::PrintSelf(ostream& os, vtkIndent indent)
 
 //------------------------------------------------------------------------------
 bool vtkVolumeTexture::AreDimensionsValid(
-  vtkTextureObject* texture, int const width, int const height, int const depth)
+  vtkTextureObject* texture, int width, int height, int depth)
 {
   int const maxSize = texture->GetMaximumTextureSize3D();
   if (width > maxSize || height > maxSize || depth > maxSize)
@@ -934,8 +936,8 @@ bool vtkVolumeTexture::AreDimensionsValid(
 }
 
 //------------------------------------------------------------------------------
-bool vtkVolumeTexture::SafeLoadTexture(vtkTextureObject* texture, int const width, int const height,
-  int const depth, int numComps, int dataType, void* dataPtr)
+bool vtkVolumeTexture::SafeLoadTexture(vtkTextureObject* texture, int width, int height, int depth,
+  int numComps, int dataType, void* dataPtr)
 {
   if (!AreDimensionsValid(texture, width, height, depth))
   {
@@ -1230,3 +1232,4 @@ vtkDataArray* vtkVolumeTexture::GetLoadedScalars()
 {
   return this->Scalars;
 }
+VTK_ABI_NAMESPACE_END
